//<GUI menuname="Easy Formatting" tooltip="Easy Auto Formating for 1D/2D Spectra"/>
/******************************************************************************************************
Copyright (C) 2009 Mestrelab Research S.L. All rights reserved.
This script is created for automatically formatting the currently opened spectrum. 
Note that before running this script for the first time, you should make sure the path to the logo file is 
correct on your computer (see the first statement in function EF1()). 

Designed to use the default setting of parameters, so that the user does not have to customize the 
parameter display before running this script. Hopefully in the future release more 'default' parameters
will be added, and then more parameters will be added to display using this script. 

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.

Date: Feb. 12, 2009 by Chen Peng
*****************************************************************************************************/
function easyFormatting()
{
// Company Logo file. Change this to your actual file path: 
	const logoFile = "C:/logo.jpg";

// Load the logo file: 
	serialization.open(logoFile);

// Now change the layout of the objects: 
	var w = new DocumentWindow(Application.mainWindow.activeWindow());
	var p = new Page(w.curPage());
	var c = p.itemCount();
	var i = 0;
	var itemArray = new Array(c);

	var specDim = 0; 
	var specID = -1; 		// If there are multiple spectra, only take care of the largest one. 
	var specSize = 0; 
	for(i=0; i<c; i++)
	{
		itemArray[i] = p.item(i);
		if( itemArray[i].name == "NMR Spectrum")
		{
			var tmpSize = itemArray[i].width+ itemArray[i].height; 
			if( tmpSize > specSize)
			{
				var nmrSpec = new NMRSpectrum(itemArray[i]);
				if( nmrSpec.isValid())
				{
					specSize = tmpSize; 
					specDim = nmrSpec.dimCount(); 
					specID = i; 
				}
			}
		}
	}
	if( specDim == 0 || specID < 0) 
	 	return; 

// If the multiplet report is there, format it: 
	var iTextBox = -1; 
	for( i = 0; i<itemArray.length; i++ )
	{
		var item = itemArray[i];
		if( iTextBox <0 && item.name == "Text" && item.type == "Report Special" && item.subtype.substring(0,10)=="Multiplets")
		{
			item.left = p.left+10;
			item.right = p.right-10;
			item.top = p.bottom -30;
			item.bottom = p.bottom -5;
			iTextBox = i;
		}
	}

// Now start the formatting: 
	for( i = 0; i<itemArray.length; i++ )
	{
		var item = itemArray[i];
		var width = item.width;
		var height = item.height;

		switch(item.name)
		{
			case "NMR Spectrum":
				if( i != specID)			// ignore small ones if multiple spectral boxes.  
					break; 
				var nmrSpc = new NMRSpectrum(item);
				var proc = new NMRProcessing(nmrSpc.proc);

				var specType = "b"; 
				if(nmrSpc.originalFormat.indexOf("Varian",0) >= 0)	specType="v";
				if(nmrSpc.originalFormat.indexOf("JEOL",0) >= 0) 	specType="j";
				if(nmrSpc.originalFormat.indexOf("JCAMP",0) >= 0) 	specType="x";

//				nmrSpc.horZoom(-0.5, 10); 			// chemical shift range to display
				nmrSpc.left = p.left +10
				nmrSpc.right = p.right-62;
				nmrSpc.top = p.top+20;
				nmrSpc.bottom = p.bottom - 30;
				if( iTextBox <0)
					nmrSpc.bottom = p.bottom;

// Display the title:  (This assumes that the user will fill in the proper contents in the title parameter). 
				var title2Use = nmrSpc.getParam("Title");
				tit = draw.text("<font style=\"font-size: 10pt; font-family: Arial\"><b>" + title2Use + "</b></font>", "", "", true);
				if(tit)
				{
// Location of the title: 
// 	Put the title to the left: 
					tit.left = p.left + 20;
// 	Put the title in the middle: 
//					tit.left = p.left + 0.5*p.width - 0.5*tit.width;
					tit.top = p.top+10;
					tit.update();
				}

// Display the "default" spectral parameters: 
				if( specDim == 1) 
					var params = "<b>Acquisition Parameters</b><br/>";
				else
					var params = "<b>Acquisition Parameters [F1, F2]</b><br/>";
				params += "<b>NAME: </b>" + Shorten_filename(nmrSpc.getParam("Data File Name"));
//				params += "<br/><b>Title: </b>" + nmrSpc.getParam("Title");
				var str = nmrSpc.getParam("Origin");
				var len = str.length;
				if( len > 20)
				{
					str = str.substring(0, 21) + "..."; 
				}
				params += "<br/><b>Origin: </b>" + str;
				params += "<br/><b>Owner: </b>" + nmrSpc.getParam("Owner");
				params += "<br/><b>Solvent: </b>" + nmrSpc.getParam("Solvent");
				params += "<br/><b>PULPROG: </b>" + nmrSpc.getParam("Pulse Sequence");
				params += "<br/><b>DATE: </b>" + nmrSpc.getParam("Acquisition Date");
				var d = new Date();
	//			params += "<br/><b>Current Date: </b>" + d.toLocaleString();

				d = nmrSpc.getParam("Temperature"); 
				params += "<br/><b>TE: </b>" + d;
				if( specType == "b")
					params += " K";
				else if( specType == "v" || specType == "j")
					params += " C"; 
				else if( d > 300) 
					params += " K";
				else
					params += " C"; 

				params += "<br/><b>NS: </b>" + nmrSpc.getParam("Number of Scans");
				d = nmrSpc.getParam("Spectrometer Frequency");
				params += "<br/><b>SFO1: </b>" + d;
				if( d > 100000000)
					params += " Hz";
				else
					params += " MHz";
				params += "<br/><b>SWH: </b>" + nmrSpc.getParam("Spectral Width") + " Hz";
//				params += "<br/><b>Lowest Freq.: </b>" + nmrSpc.getParam("Lowest Frequency") + " Hz";
				params += "<br/><b>NUC: </b>" + nmrSpc.getParam("Nucleus");
				params += "<br/><b>TD: </b>" + nmrSpc.getParam("Acquired Size");
// Processing Parameters: 
				params += "<br/><br/><b>F2 - Processing Parameters</b>" ;				

				var paraname = "Spectral Size[1]"; 
				if(specDim > 1) paraname = "Spectral Size[2]"; 		
				params += "<br/><b>SI: </b>" + nmrSpc.getParam(paraname);

				if(proc.getParameter("DC.Apply") == true)
					params += "<br/><b>DC: </b>" + proc.getParameter("DC.Value"); 

				paraname = "Lp[1]."; 
				if(specDim > 1) paraname = "Lp[2]."; 		// For 2D, Lp[2] is that of F2 - first dimension. 
				if(proc.getParameter(paraname+"Forward.Apply") == true || proc.getParameter(paraname+"Backward.Apply") == true )
				{
					params += "<br/><b>LP: </b>"; 
					if(proc.getParameter(paraname+"Backward.Apply") == true)
						params += "Backward, from " + proc.getParameter(paraname+"Backward.First") + " to " + proc.getParameter(paraname+"Backward.Last"); 
					if(proc.getParameter(paraname+"Forward.Apply") == true)
						params += "Forward, from " + proc.getParameter(paraname+"Forward.First") + " to " + proc.getParameter(paraname+"Forward.Last"); 
					params += "<br/>" + proc.getParameter(paraname+"method") + " Basis Pts="+ proc.getParameter(paraname+"BasisPoints") + " Coef=" + proc.getParameter(paraname+"Coefficients"); 
				}

				paraname = "apodization[1]."; 
				if( specDim > 1) paraname = "apodization[2]."; 
				if(proc.getParameter(paraname+"Apply") == true)
				{
					if( specDim == 1) 
						params += reportApodization(1, proc);
					else
						params += reportApodization(2, proc);
				}
						
				params += "<br/><b>FT: </b>";
				paraname = "Ft[1]."; 
				if( specDim > 1) paraname = "Ft[2]."; 
				if( proc.getParameter(paraname+"Protocol") != "None")  params+= proc.getParameter(paraname+"Protocol"); 
				if( proc.getParameter(paraname+"Hyper")==true) params+= " Hyper";
				if( proc.getParameter(paraname+"Invert")==true) params+= " Invert";
				if( proc.getParameter(paraname+"Quadrature")==true) params+= " Quadrature";
				if( proc.getParameter(paraname+"RealFT")==true) params+= " Real";
				
				paraname = "Pc[1]."; 
				if( specDim > 1) paraname = "Pc[2]."; 
				params += "<br/><b>Phase: </b>" + proc.getParameter(paraname+"Method"); 
				if( proc.getParameter(paraname+"Method") == "Manual")
				{
					var ph0 = proc.getParameter(paraname+"Ph0")*180.0/Math.PI;
					var ph1 = proc.getParameter(paraname+"Ph1")*180.0/Math.PI;
					params += "<br/><b>Ph0: </b>" + ph0.toFixed(2) + "<br/><b>Ph1: </b>"  + ph1.toFixed(2); 
				}
				
				paraname = "Bc[1]."; 
				if( specDim > 1) paraname = "Bc[2]."; 
				if(proc.getParameter(paraname+"Apply") == true)
				{
					params += "<br/><b>Baseline: </b>" + proc.getParameter(paraname+"algorithm"); 
				}

				if( specDim > 1)
				{
					params += "<br/><br/><b>F1 - Processing Parameters</b>" ;
					params += "<br/><b>SI: </b>" + nmrSpc.getParam("Spectral Size[1]");
					if(proc.getParameter("Lp[1].Forward.Apply") == true || proc.getParameter("Lp[1].Backward.Apply") == true )
					{
						params += "<br/><b>LP: </b>"; 
						if(proc.getParameter(Lp[1]+"Backward.Apply") == true)
							params += "Backward, from " + proc.getParameter(Lp[1]+"Backward.First") + " to " + proc.getParameter(Lp[1]+"Backward.Last"); 
						if(proc.getParameter(Lp[1]+"Forward.Apply") == true)
							params += "Forward, from " + proc.getParameter(Lp[1]+"Forward.First") + " to " + proc.getParameter(Lp[1]+"Forward.Last"); 
						params += "<br/>" + proc.getParameter(Lp[1]+"method") + " Basis Pts="+ proc.getParameter(Lp[1]+"BasisPoints") + " Coef=" + proc.getParameter(Lp[1]+"Coefficients");
					}

					if(proc.getParameter("apodization[1].Apply") == true)
						params += reportApodization(1, proc);
					params += "<br/><b>FT: </b>";
					if( proc.getParameter("FT[1].Protocol") != "None")  params+= proc.getParameter("FT[1].Protocol"); 
					if( proc.getParameter("FT[1].Hyper")==true) params+= " Hyper";
					if( proc.getParameter("FT[1].Invert")==true) params+= " Invert";
					if( proc.getParameter("FT[1].Quadrature")==true) params+= " Quadrature";
					if( proc.getParameter("FT[1].RealFT")==true) params+= " Real";
					params += "<br/><b>Phase: </b>" + proc.getParameter("Pc[1].Method"); 
					if( proc.getParameter("Pc[1].Method") =="Manual")
					{
						var ph0 = proc.getParameter("Pc[1].Ph0")*180.0/Math.PI;
						var ph1 = proc.getParameter("Pc[1].Ph1")*180.0/Math.PI;
						params += "<br/><b>Ph0: </b>" + ph0.toFixed(2) + "<br/><b>Ph1: </b>"  + ph1.toFixed(2); 
					}
					if(proc.getParameter("Bc[1].Apply") == true)
					{
						params += "<br/><b>Baseline: </b>" + proc.getParameter("Bc[1].algorithm"); 
					}
				}
// location of the parameter box: 
				var par = draw.text("<font style=\"font-size: 8pt; font-family: Courier New\">"+params+"</font>", "","",true);
				par.left = p.right - 62;
				par.right = p.right;
				par.top = p.top+ 25;	
				par.bottom = p.bottom;				
			  
			break;

// location of the Mol. Structure: 
			case "Molecule":
				item.left = p.left+15;
				item.top = p.top+65;
			break;

// location of the company logo: 
			case "Image":
				item.left = p.right - item.width;
				item.top = p.top+5;				
			break;
		}

	}
	w.update();
}

function reportApodization(dim, p)
{
	var apname = "Apodization["+dim+"]."; 
	var result="";
	if(p.getParameter(apname+"Apply") == true)
	{
		if(p.getParameter(apname+"Exp.apply") == true)
			result+="<br/><b>LB: </b>" +p.getParameter(apname+"Exp.Value").toFixed(2) + " Hz"; 
		if(p.getParameter(apname+"ConDif.Apply") == true)
			result+="<br/><b>Convolution Diff.: </b>"+ p.getParameter(apname+"ConDif.Value1") + " " + p.getParameter(apname+"ConDif.Value2") + " "+ p.getParameter(apname+"ConDif.Value3").toFixed(2); 
		if(p.getParameter(apname+"Gauss.Apply") == true)
			result+="<br/><b>Gauss: </b>" + p.getParameter(apname+"Gauss.Value").toFixed(2); 
		if(p.getParameter(apname+"Han.Apply") == true)
			result+="<br/><b>Hanning: </b>"+ p.getParameter(apname+"Han.Value").toFixed(2); 
		if(p.getParameter(apname+"Lg.Apply") == true)
			result+="<br/><b>Lg: </b>"+ p.getParameter(apname+"Lg.Value").toFixed(2); 
		if(p.getParameter(apname+"Lr.Apply") == true)
			result+="<br/><b>Linear Ramp:</b> yes "; 
		if(p.getParameter(apname+"Sine.Apply") == true)
			result+="<br/><b>Sine: </b>"+ p.getParameter(apname+"Sine.Value").toFixed(2); 
		if(p.getParameter(apname+"Sine2.Apply") == true)
			result+="<br/><b>Sine2: </b>"+ p.getParameter(apname+"Sine2.Value").toFixed(2); 
		if(p.getParameter(apname+"Traf.Apply") == true)
			result+="<br/><b>Traf: </b>"+ p.getParameter(apname+"Traf.Value").toFixed(2); 
		if(p.getParameter(apname+"Trap.Apply") == true)
			result+="<br/><b>Trap: </b>"+ p.getParameter(apname+"Trap.Value").toFixed(2); 
		if(p.getParameter(apname+"FP.Apply") == true)
			result+="<br/><b>First Point: </b>" + p.getParameter(apname+"FP.Value").toFixed(2); 
	}
	return result; 
}

//shorten the path name to <= 20 with some folder names retained: 
function Shorten_filename(aFileName)
{
	var found = -1;
	var max = 20; 
	var len = 0; 
	var num = 0; 
// Search for the "/" starting from the end:
	for(var i=aFileName.length-1; i >= 0; i--)
	{
		var c = aFileName[i];
		len ++; 
		if( c == '\\' || c=='/')
		{	
			num++; 
			if( num > 0 && len >= max)	
			{
				found=i;
				break;
			}
		}
	}
	return aFileName.substring(found+1, aFileName.length); 
}