/******************************************************************************************************
Copyright (C) 2012 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

/*globals BaseConverter, NMRSpectrum, File, TextStream, settings, strings, Env, Dir, mainWindow, Application*/
/*jslint plusplus: true, indent: 4*/

function Custom1DCsvConverter() {
	"use strict";
	BaseConverter.call(this);
	this.exportFileFilter = "NMR 1D Custom CSV (*.csv *.txt)";
}

Custom1DCsvConverter.prototype = new BaseConverter();

Custom1DCsvConverter.prototype.wantsExport = function (aPageItems) {
	"use strict";
	var i, spec;

	for (i = 0; i < aPageItems.length; i++) {
		spec = new NMRSpectrum(aPageItems[i]);
		if (spec.isValid() && spec.dimCount === 1) {
			return true;
		}
	}
	return false;
};

Custom1DCsvConverter.prototype.exportItems = function (aPageItems, aFilename) {
	"use strict";
	var format, formats, decimals,
		dlg = Application.loadUiFile("ricares:WCustom1DCsvConverter.ui");

	dlg.widgets.sbDecimalPlaces.value = settings.value("Custom1DCsvConverter/DecimalsPlaces", dlg.widgets.sbDecimalPlaces.value);
	dlg.widgets.cmbFormat.items = settings.value("Custom1DCsvConverter/Formats", ["{ppm}{tab}{real}{tab}{imag}", "{ppm}{tab}{real}", "{hz}{tab}{real}{tab}{imag}", "{hz}{tab}{real}"]).map(function (aItem) {return aItem.replace(/\[/g, "{").replace(/\]/g, "}"); });
	dlg.widgets.cbRightToLeft.checked = settings.value("Custom1DCsvConverter/RightToLeft", dlg.widgets.cbRightToLeft.checked);

	mainWindow.restoreCursor();
	if (!dlg.exec()) {
		return 0;
	}

	format = dlg.widgets.cmbFormat.currentText;
	decimals = dlg.widgets.sbDecimalPlaces.value;
	settings.setValue("Custom1DCsvConverter/DecimalsPlaces", decimals);
	settings.setValue("Custom1DCsvConverter/RightToLeft", dlg.widgets.cbRightToLeft.checked);
	formats = dlg.widgets.cmbFormat.items.filter(function (aElement, aIndex) {return aElement !== format && aIndex <= 8; });
	formats.unshift(format);
	settings.setValue("Custom1DCsvConverter/Formats", formats);

	mainWindow.setWaitCursor();
	try {
		Custom1DCsvConverter.formattedExport(aPageItems, aFilename, format.replace(/\{tab\}/g, "\t"), decimals, dlg.widgets.cbRightToLeft.checked);
	} finally {
		mainWindow.restoreCursor();
	}
};

Custom1DCsvConverter.formattedExport = function (aPageItems, aFilename, aFormat, aDecimals, aRightToLeft) {
	"use strict";
	var file, strm, hz, dHz, i, pt, dPt, endPt, ppm, dPpm, spec,
		mapObj = {};

	try {
		file = new File(aFilename);
		if (file.open(File.WriteOnly)) {
			strm = new TextStream(file);
			for (i = 0; i < aPageItems.length; i++) {
				spec = new NMRSpectrum(aPageItems[i]);
				if (spec.isValid() && spec.dimCount === 1) {
					if (aRightToLeft) {
						hz = spec.hz();
						dHz = spec.scaleWidth() / spec.count();
						pt = spec.count() - 1;
						dPt = -1;
						endPt = -1;
					} else {
						hz = spec.hz() + spec.scaleWidth();
						dHz = -spec.scaleWidth() / spec.count();
						pt = 0;
						dPt = 1;
						endPt = spec.count();
					}
					ppm = hz / spec.frequency();
					dPpm = dHz / spec.frequency();
					while (pt !== endPt) {
						mapObj.hz = hz.toFixed(aDecimals);
						mapObj.ppm = ppm.toFixed(aDecimals);
						mapObj.pts = pt;
						mapObj.real = spec.real(pt).toFixed(aDecimals);
						mapObj.imag = spec.imag(pt).toFixed(aDecimals);
						strm.writeln(aFormat.formatMap(mapObj));
						hz += dHz;
						ppm += dPpm;
						pt += dPt;
					}
					break;
				}
			}
		}
	} finally {
		file.close();
	}
};

Env.Converters.push(new Custom1DCsvConverter());

Custom1DCsvConverter.Custom1DCsvConverterTest = function () {
	"use strict";
	var retVal = -1,
		cnv = new Custom1DCsvConverter(),
		pageItems = mainWindow.activeDocument.selection();

	if (cnv.wantsExport(pageItems)) {
		retVal = cnv.exportItems(pageItems, Dir.home() + "/Custom1DCsv.txt");
	}
	return retVal;
};
