/******************************************************************************************************
Copyright (C) 2012 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

/*globals BaseConverter, NMRSpectrum, File, TextStream, settings, strings, Env, Dir, mainWindow*/
/*jslint plusplus: true, indent: 4*/

/** Default constructor of the converter
 * Defines the file filter
*/
function PeakListConverter() {
	"use strict";
	BaseConverter.call(this);
	this.exportFileFilter = "NMR 1D Peak List (*.csv *.txt)";
}

PeakListConverter.prototype = new BaseConverter();

PeakListConverter.prototype.wantsExport = function (aPageItems) {
	"use strict";
	var i, spec;

	for (i = 0; i < aPageItems.length; i++) {
		spec = new NMRSpectrum(aPageItems[i]);
		if (spec.isValid() && spec.dimCount === 1 && spec.peaks().count) {
			return true;
		}
	}
	return false;
};

PeakListConverter.prototype.exportItems = function (aPageItems, aFilename) {
	"use strict";
	var strm, i, j, s, spec, peaks, peak, file, oldCurSpecIndex;

	try {
		file = new File(aFilename);
		if (file.open(File.WriteOnly)) {
			strm = new TextStream(file);
			strm.precision = settings.value("PeakListConverter/ExportDecimals", 6);
			for (i = 0; i < aPageItems.length; i++) {
				spec = new NMRSpectrum(aPageItems[i]);
				if (spec.isValid() && spec.dimCount === 1) {
					oldCurSpecIndex = spec.curSpecIndex;
					for (s = 0; s < spec.specCount; s++) {
						spec.curSpecIndex = s;
						strm.write(s + 1, '\t', Str.linifyAndQuote(spec.title));
						peaks = spec.peaks();
						peaks.sort(false);
						for (j = 0; j < peaks.count; j++) {
							peak = peaks.at(j);
							strm.write('\t', peak.delta(), '\t', peak.intensity);
						}
						strm.writeln();
					}
					spec.curSpecIndex = oldCurSpecIndex;
				}
			}
		}
	} finally {
		file.close();
	}
	return 0;
};

Env.Converters.push(new PeakListConverter());

PeakListConverter.PeakListConverterTest = function () {
	"use strict";
	var retVal = -1,
		cnv = new PeakListConverter(),
		pageItems = mainWindow.activeDocument.selection();

	if (cnv.wantsExport(pageItems)) {
		retVal = cnv.exportItems(pageItems, Dir.home() + "/PeakList.txt");
	}
	return retVal;
};
