/******************************************************************************************************
Copyright (C) 2009 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/ 
// This function defines JACS-like multiplet report.
// To customize report, edit this class or implement another one.
function JACSMultipletReporter()
{
	MultipletReporter.call(this);

	this.name = new String("JACS");

	this.onlyElementName=false;
	// Define font size and font family
	this.font = "<font style=\"font-size: 10pt; font-family: Times New Roman\">";

	this.nucleusTemplate="%1";
	// Report header. %1 will be raplaced with nucleusString, %2  with frequency, %3 with solvent
	this.header = "%1 NMR (%2 MHz, %3) &delta; ";

	// Multiplet templates. %1 - delta, %2 - category, %3 - nH
	this.reportRange = true; // set to true to get multiplet range instead of delta.
	this.withoutJsTemplate = " %1 (%2, %3%NUCLEUS%)"; // multiplet template without J's
	this.withJsTemplate = " %1 (%2, %4 Hz, %3%NUCLEUS%)"; //  multiplet template with J's
	this.rangeTemplate = "%1 &ndash; %2";

	// J's list template. %1 - list of J's
	this.jListTemplate = "<i>J</i> = %1";

	this.jPrecision = 1; // J's precision
	this.deltaPrecision = 2; // delta precision
	this.mSeparator = ", "; // multiplet separator
	this.jSeparator = ", "; // J's separator

	this.start = this.font;
	this.end = ".</font>";
}

JACSMultipletReporter.prototype = new MultipletReporter();
JACSMultipletReporter.prototype.toString = function() { return "JACSMultipletReporter()"; }

// This function defines Angewandte-like multiplet report.
// To customize report, edit this function or implement another one.
function AngewandteMultipletReporter()
{
	MultipletReporter.call(this);

	this.name = new String("Angewandte");

	this.onlyElementName=false;
	// Define font size and font family
	this.font = "<font style=\"font-size: 10pt; font-family: Times New Roman\">";

	this.nucleusTemplate="%1";
	// Report header. %1 will be raplaced with nucleusString, %2  with frequency, %3 with solvent
	this.header = "%1 NMR (%2 MHz, %3) &delta; = ";

	// Multiplet templates. %1 - delta, %2 - category, %3 - nH
	this.reportRange = true; // set to true to get multiplet range instead of delta.
	this.withoutJsTemplate = " %1 (%2, %3%NUCLEUS%)"; // multiplet template without J's
	this.withJsTemplate = " %1 (%2, %4, %3%NUCLEUS%)"; //  multiplet template with J's
	this.rangeTemplate = "%1 &ndash; %2";

	// J's list template. %1 - list of J's
	this.jListTemplate = "<i>J</i>=%1";

	this.jPrecision = 1; // J's precision
	this.deltaPrecision = 2; // delta precision
	this.mSeparator = ", "; // multiplet separator
	this.jSeparator = ", "; // J's separator

	this.start = this.font;
	this.end = ".</font>";
}

AngewandteMultipletReporter.prototype = new MultipletReporter();
AngewandteMultipletReporter.prototype.toString = function() { return "AngewandteMultipletReporter()"; }

// This function defines RSC-like multiplet report.
// To customize report, edit this function or implement another one.
function RSCMultipletReporter()
{
	MultipletReporter.call(this);

	this.name = new String("RSC");

	this.onlyElementName=true;
	// Define font size and font family
	this.font = "<font style=\"font-size: 10pt; font-family: Times New Roman\">";

	//%1 will be replaced with nucleus
	this.nucleusTemplate="<sub>%1</sub>";
	// Report header. %1 will be raplaced with the value of nucleusTemplate, %2  with frequency, %3 with solvent
	this.header = "&delta; %1 (%2 MHz, %3) ";

	// Multiplet templates. %1 - delta, %2 - category, %3 - nH
	this.reportRange = true;  // set to true to get multiplet range instead of delta.
	this.withoutJsTemplate = " %1 (%3 %NUCLEUS%, %2)"; // multiplet template without J's
	this.withJsTemplate = " %1 (%3 %NUCLEUS%, %2, %4)"; //  multiplet template with J's
	this.rangeTemplate = "%1 &ndash; %2";

	// J's list template. %1 - list of J's
	this.jListTemplate = "<i>J</i> %1";

	this.jPrecision = 1; // J's precision
	this.deltaPrecision = 2; // delta precision
	this.mSeparator = ", "; // multiplet separator
	this.jSeparator = ", "; // J's separator

	this.start = this.font;
	this.end = ".</font>";
}

RSCMultipletReporter.prototype = new MultipletReporter();
RSCMultipletReporter.prototype.toString = function() { return "RSCMultipletReporter()"; }

// Warning! Do not delete the below function, do not rename it, it is called from MNova!
// Changing its contents is allowed. In particular, edit the type of multiplet reporter
function automaticSingleMultipletReport(aMultiplet, aSpectrum)
{
	var reporter = MultipletReporter.getDefaultReporter("JACS");
	var showMAsRanges = settings.value( MultipletReporter.useRangeSettings, true );
	reporter.reportRange = showMAsRanges;
	return reporter.reportSingleMultiplet(aMultiplet, aSpectrum);
}


// Warning! Do not delete the below function, do not rename it, it is called from MNova!
// Changing its contents is allowed. In particular, edit the type of multiplet reporter
function automaticMultipletReport()
{
	return reportMultipletsUser(false);
}

// Warning! Do not delete the below function, do not rename it, it is called from MNova!
// Changing its contents is allowed. In particular, edit the function to avoid showing the dialog.
function reportMultipletsUser(createItem)
{		
	if( Env.ReportMultipletsUser != undefined )
		return Env.ReportMultipletsUser(createItem);
	else
		return MultipletReporter.reportMultipletsUser(createItem);
}

function customizeReportMultipletsUser()
{
	if( Env.ReportMultipletsUserSetup != undefined )
		return Env.ReportMultipletsUserSetup();
	else
	{
		if( Env.DefaultMultipletReporter != undefined )
			MessageBox.information ( "Using Default Multiplet Reporter: "+ Env.DefaultMultipletReporter.toString(), MessageBox.Ok, MessageBox.NoButton, MessageBox.NoButton, "Information" );
		else
		{
			var diag = new MultipletReportDialog(Env.MultipletReportersArray, true);
			diag.exec();
		}
	}
}

// MultipletReporter is base class for JACSMultipletReporter, AngewandteMultipletReporter, RSCMultipletReporter.
// To customize reports, edit these classes or implement another ones inherited from MultipletReporter.
function MultipletReporter()
{
	this.name = new String("Custom");
	// fields to be filled in in the child classes
	this.font = new String;
	this.header = new String;
	this.nucleusTemplate = new String;
	this.onlyElementName = false;
	this.withoutJsTemplate= new String;
	this.withJsTemplate= new String;
	this.rangeTemplate= new String;
	this.reportRange = true;
	this.jListTemplate= new String;
	this.jPrecision = 1;
	this.deltaPrecision = 2;
	this.mSeparator= new String;
	this.jSeparator= new String;
	this.start= new String;
	this.end= new String;
	this.sortJAscending = false;
}

MultipletReporter.prototype.toString = function() { return "MultipletReporter()"; }
MultipletReporter.templateSettings = new String("Multiplet Report/Template");
MultipletReporter.useRangeSettings = new String("Multiplet Report/Use Ranges");

MultipletReporter.reportMultipletsUser = function(createItem)
{
	var multipletTable = new MultipletTable(nmr.multipletTable()); // Get table of multiplets
	var spectrum;
	if(multipletTable.isValid())
		spectrum = new NMRSpectrum(multipletTable.spectrum()); // Get spectrum currently shown in the table of multiplets
	else
		spectrum = new NMRSpectrum(nmr.activeSpectrum());

	if( spectrum.isValid() )
	{
		var reporter = MultipletReporter.getDefaultReporter("JACS");
		var showMAsRanges = settings.value( MultipletReporter.useRangeSettings, true );
		reporter.reportRange = showMAsRanges;
		return reporter.report(spectrum, createItem);
	}
	else
		return undefined;
}

MultipletReporter.getReporterByName = function(aName)
{
	for( var i = 0; i < Env.MultipletReportersArray.length; i++ )
	{
		if( Env.MultipletReportersArray[i].name == aName )
			return Env.MultipletReportersArray[i];
	}
	return undefined;
}

MultipletReporter.getDefaultReporter = function(aDefValue)
{
	if( Env.DefaultMultipletReporter != undefined )
	{
		var retVal = MultipletReporter.getReporterByName(Env.DefaultMultipletReporter);
		if( retVal != undefined )
			return retVal;
		else
			return MultipletReporter.getReporterByName(aDefValue);
	}
	else
	{
		var defaultName = settings.value( MultipletReporter.templateSettings, aDefValue );
		var retVal = MultipletReporter.getReporterByName(defaultName);
		if( retVal != undefined )
			return retVal;
		else
			return MultipletReporter.getReporterByName(aDefValue);
	}
}

MultipletReporter.prototype.reportSingleMultiplet = function(multiplet,spectrum)
{
	var jList = new JList(multiplet.jList());
	jList.sort(this.sortJAscending);
	var jCount = jList.length;

	var mTemplate = new String();
	if (jCount > 0)
		mTemplate = this.withJsTemplate;
	else
		mTemplate = this.withoutJsTemplate;
	const c1 = "%2";

	var shiftStr = new String();
	if( this.reportRange && multiplet.category=="m" )
		shiftStr = this.rangeTemplate.replace("%1", multiplet.rangeMax.toFixed(this.deltaPrecision)).replace("%2", multiplet.rangeMin.toFixed(this.deltaPrecision));
	else
		shiftStr = c1.replace("%2", multiplet.delta.toFixed(this.deltaPrecision));
	var nucleusStr = spectrum.nucleus(false);
	var numberRegExp = new RegExp("[0-9]*");
	nucleusStr = nucleusStr.replace(numberRegExp,"");
	if(nucleusStr != "H")
		mTemplate=mTemplate.replace("%3","").replace("%NUCLEUS%","").replace("( , ", "(").replace(", )", ")").replace(", ,",",");

	var multStr = mTemplate.replace("%1", shiftStr).replace("%2", multiplet.category).replace("%3", multiplet.nH).replace("%NUCLEUS%",nucleusStr);
	//print("multStr "+multStr);

	if (jCount > 0)
	{
		var js = new String();
		for(var ij = 0; ij < jCount; ij++)
		{
			//print( ij );
			if (js.length != 0)
				js += this.jSeparator;
			js += c1.replace("%2", jList.at(ij).toFixed(this.jPrecision));
			//print("js "+ js);
		}
		multStr = multStr.replace("%4", this.jListTemplate.replace("%1", js));
		//print("multStr "+multStr);
	}
	return multStr;
}

MultipletReporter.prototype.report = function (spectrum, createItem)
{
	var multiplets = new Multiplets(spectrum.multiplets()); // get multiplets from spectrum
	multiplets.sort(false);
	const count = multiplets.count;
	if (!count) // if no multiplets then exit
		return "";

	// Generate HTML report string
	var nucleusString = new String();
	if( this.onlyElementName )
		nucleusString = removeNumbers(spectrum.nucleus());
	else
		nucleusString = spectrum.nucleus(true);
	nucleusString = this.nucleusTemplate.replace("%1", nucleusString);
	var result = "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><title>Multiplet Report</title></head><body>" + this.start + this.header.replace("%1", nucleusString).replace("%2", Math.round(spectrum.frequency())).replace("%3", solventStr2Html(spectrum.solvent));
	for(var i = 0; i < count; i++)
	{
		var multiplet = new Multiplet(multiplets.at(i));
		if (multiplet.category == "")
			continue;
		result += this.reportSingleMultiplet(multiplet,spectrum);

		if (i < count-1)
			result += this.mSeparator;
	}
	result += this.end + "</body></html>";

	if( createItem )
	{
		var oldLTCount = mainWindow.activeWindow().itemCount("Layout Template");
		var textItem = draw.text(result, "Report Special", "Multiplets,"+spectrum.nucleus(), true);
		if (oldLTCount == mainWindow.activeWindow().itemCount("Layout Template"))
		{
			textItem.left = spectrum.left;
			textItem.right = (spectrum.left+spectrum.right)/2.0;
			textItem.top = spectrum.top;
			textItem.bottom = spectrum.top + (spectrum.bottom-spectrum.top)/4.0;
			mainWindow.activeWindow().update();
		}
	}

	return result;
}

//Function to create reports for the active spectrum
function reportMultiplets(reporter)
{
	var spectrum = new NMRSpectrum(nmr.activeSpectrum());
	reporter.report(spectrum, true);
}

Env.MultipletReportersArray.push( new JACSMultipletReporter() );
Env.MultipletReportersArray.push( new RSCMultipletReporter() );
Env.MultipletReportersArray.push( new AngewandteMultipletReporter() );

function MultipletLabelReporter()
{
	this.name = "Custom Reporter";
}
MultipletLabelReporter.prototype.toString = function() { return this.name; }

function MNovaMultipletLabelReporter()
{
	MultipletLabelReporter.call(this);

	this.name="MNova Default";
}
MNovaMultipletLabelReporter.prototype = new MultipletLabelReporter();
MNovaMultipletLabelReporter.prototype.report = function(aMultiplet)
{
	var multLabel = new String("%1 (%2)\n%3");
	multLabel = multLabel.replace("%1",aMultiplet.name).replace("%2",aMultiplet.category).replace("%3",aMultiplet.delta.toFixed(2));
	return multLabel;
}

function MNovaMultipletLabelReporterWithJ(aMultiplet)
{
	MultipletLabelReporter.call(this);

	this.name = "MNova Default with J's";
}
MNovaMultipletLabelReporterWithJ.prototype = new MultipletLabelReporter();
MNovaMultipletLabelReporterWithJ.prototype.report = function(aMultiplet)
{
	var multLabel = new String("%1 (%2)\n%3");
	multLabel = multLabel.replace("%1",aMultiplet.name).replace("%2",aMultiplet.category).replace("%3",aMultiplet.delta.toFixed(2));
	var jList = aMultiplet.jList(true);
	if( jList.length != 0 )
	{
		multLabel += "\n%4";
		var js = new String("J(%1)");
		var jListString = new String();
		for(var ij = 0; ij < jList.length; ij++)
		{
			if (jListString.length != 0)
				jListString += ", ";
			jListString += jList.at(ij).toFixed(2);
		}
		multLabel = multLabel.replace("%4", js.replace("%1",jListString));
	}
	return multLabel;
}

function MNovaMultipletLabelReporterName()
{
	MultipletLabelReporter.call(this);

	this.name="Multiplet Name Only";
}
MNovaMultipletLabelReporterName.prototype = new MultipletLabelReporter();
MNovaMultipletLabelReporterName.prototype.report = function(aMultiplet)
{
	return aMultiplet.name;
}

Env.MultipletLabelFunctions.push( new MNovaMultipletLabelReporter() );
Env.MultipletLabelFunctions.push( new MNovaMultipletLabelReporterWithJ() );
Env.MultipletLabelFunctions.push( new MNovaMultipletLabelReporterName() );

function getLabelReporter(aName)
{
	for( var i = 0; i<Env.MultipletLabelFunctions.length; i++ )
	{
		if( Env.MultipletLabelFunctions[i].name == aName )
			return Env.MultipletLabelFunctions[i];
	}
	return undefined;
}

function getLabelReportNameList()
{
	var namesList = new Array();
	for( var i = 0; i<Env.MultipletLabelFunctions.length; i++ )
		namesList.push(Env.MultipletLabelFunctions[i].name);
	return namesList;
}

