/******************************************************************************************************
Copyright (C) 2009 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided 
AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, 
MERCHANTABILITY AND FITNESSFOR A PARTICULAR PURPOSE.
*****************************************************************************************************/ 

// This file contains several functions for accessing object properties. It is needed by DBPlugin.

function accessFunctionsTest(aUuid)
{
	if( aUuid == undefined)
	{
		var dw = new DocumentWindow(mainWindow.activeWindow());
		for( var i = 0; i < dw.itemCount(); i++ ) 
		{
			print("  Item " + i + ": "+ dw.item(i));
			accessFunctionsTest(dw.item(i).uuid)
		}
		print(dw.itemCount() + " items on " + dw.pageCount() + " pages.");
	}
	else
	{
		var item = getItem(aUuid);
		if (item.name == "Molecule")
		{
			var molecule =  Molecule(item);
			print("    molFile = " + molecule.getMolfile());
			print("    molecularFormula = " + molecule.molecularFormula());
			print("    molecularWeight = " + molecule.molecularWeight());
		}
		else if (item.name == "NMR Spectrum")
		{
			var spectrum = NMRSpectrum(item);
			var paramNames = spectrum.paramNames();
			print("    " + paramNames.length +  " parameters.");
			for( var i = 0; i < paramNames.length; i++ ) 
			{
				print("      " + paramNames[i] + ": " + spectrum.getParam(paramNames[i]));
			}
			print("    nmrPeaks1D = " + getNMRPeaks(spectrum, 1));
			print("    nmrPeaks2D = " + getNMRPeaks(spectrum, 2));
			//print("    nmrMultiplets = " + getNMRMultiplets(aUuid));
			print("    nmrOriginalFormat = " + spectrum["originalFormat"]);
		}
		print("    Text = " + getText(aUuid));
	}
}

function printItems()
{
	var dw = new DocumentWindow(mainWindow.activeWindow());
	for( var i = 0; i < dw.itemCount(); i++ ) 
	{
		print(" Item " + i + ": "+ dw.item(i));
	}
	print(dw.itemCount() + " items on " + dw.pageCount() + " pages.");
}


// Warning! Do not delete or rename the functions below!


// Returns the peaklist for a spectrum as text suitable for db storage.
// Returns the peaklist iff spectrum has dimension aDim, else "0".
function getNMRPeaks(aItem, aDim)
{
	var spec = NMRSpectrum(aItem);
	var result = "0";
	if (spec.dimCount == aDim) 
	{
		var peaks = spec.peaks();
		result = "" + peaks.count;
		for (var i = 0; i < peaks.count; i++ ) 
		{
			var peak = peaks.at(i);
			result += "\n" + (i+1) + " ";
			for (var dim=1; dim <= spec.dimCount; dim++)
			{
				result += peak.delta(dim) + " " + peak.width(dim) + " ";
			}
			result += peak.intensity + " " + nmrPeakType(peak.type); 
		}
	}
	return result;
}

function nmrPeakType(aNumber)
{
	if ((aNumber) == Peak.Compound) return "C";
	if ((aNumber) == Peak.Artifact) return "A";
	if ((aNumber) == Peak.Impurity) return "I";
	if ((aNumber) == Peak.Solvent) return "S";
	if ((aNumber) == Peak.CS_Ref) return "R";
	if ((aNumber) == Peak.Quantification_Ref) return "Q";
	return "U";
}

function getNMRMultiplets(aUuid)
{
	var spec = NMRSpectrum(getItem(aUuid));
         var multiplets = spec.multiplets();
	var result = "" + multiplets.count; 
	for (var i = 0; i < multiplets.count; i++ ) 
	{
		var m = multiplets.at(i);
		result += "\n" + (i+1) 
			+ " " + m.delta 
			+ " " + m.rangeMin 
			+ " " + m.rangeMax 
			+ " " + m.nH() 
			+ " " + m.integralValue() 
			+ " " + m.category
                           + " " + m.jList().toString().replace("JList [", "[").replace(", ", ",");    
         }
	return result;
}


function getText(aItem)
{
         //Valid values: {"Arrow", "Ellipse", "Image", "Layout Template", "Mass Spectrum", "Molecule", "NMR Spectrum", "OLE Object", "Polygon", "Rectangle", "Text"}
      	if (aItem.name == "NMR Spectrum") return NMRSpectrum(aItem).title;
      	//if (item.name == "Mass Spectrum") return MassItem(item).plot().description();
	if (aItem.name == "Text") return Text(aItem).plainText;
	return "-"; 
}

function saveSVG(aUuid, aFileName)
{
	var dw = new DocumentWindow(mainWindow.activeWindow());
	// Remember current selection
	var selection = dw.selection();
	var pageSelection = dw.selectedPages();
	// Select item with id
	dw.setSelectedPages (new Array());
	dw.setSelection(dw.getItem(aUuid));
	// Save svg
	serialization.save(aFileName, "svg");
	// Set selection back to original
	dw.setSelection(selection);
	dw.setSelectedPages(pageSelection);
	return("Saved SVG of " + aUuid + " to " + aFileName + ".");
}

function getItem(aUuid)
{
	return DocumentWindow(Application.mainWindow.activeWindow()).getItem(aUuid);
}

  
