/*******************************************************************************************
Copyright (C) 2009 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*******************************************************************************************/ 

function brukerDate(aStr)
{
	var time = parseInt(aStr,10);
	if(isNaN(time))
		return aStr;
		
	var date = new Date( time*1000 ); 
	
	return date;
}

function brukerDate2(aStr, aParmDateStr, aParmTimeStr)
{
	if (aParmDateStr != undefined && aParmTimeStr != undefined && aParmDateStr.length == 8 && (aParmTimeStr.length == 5 || aParmTimeStr.length == 4))
	{
		var year = parseInt(aParmDateStr.substring(0, 4), 10);
		var month = parseInt(aParmDateStr.substring(4, 6), 10)-1;
		var day = parseInt(aParmDateStr.substring(6, 8), 10);
		var hourLen = 2;
		if (aParmTimeStr.length == 4)
			hourLen = 1;
		var hour = parseInt(aParmTimeStr.substring(0, hourLen), 10);
		var min = parseInt(aParmTimeStr.substring(hourLen+1, hourLen+3), 10);
		
		var result = new Date(year, month, day, hour, min);

		return result;
	}
	else
		return brukerDate(aStr);
}

function varianDate(aDateTime, aDateTime1, aDateTime2)
{
	if (aDateTime.length == 15 && aDateTime.charAt(8) == 'T')
	{
		var year = parseInt(aDateTime.substring(0, 4), 10);
		var month = parseInt(aDateTime.substring(4, 6), 10)-1;
		var day = parseInt(aDateTime.substring(6, 8), 10);
		var hour = parseInt(aDateTime.substring(9, 11), 10);
		var min = parseInt(aDateTime.substring(11, 13), 10);
		var sec = parseInt(aDateTime.substring(13, 15), 10);

		var result = new Date(year, month, day, hour, min, sec);

		return result;
	}
	else
		return firstNotEmptyString(aDateTime, aDateTime1, aDateTime2);
}

function jeolDate(aDate)
{
	var date = parseInt(aDate, 10);
	var year = 1990 + ((date & 0xfe000000) >> 25);
	var month = ((date & 0x01e00000) >> 21)-1;
	var day = (date & 0x001f0000) >> 16;
	var seconds = date & 0x0000ffff;

	var result = new Date(year, month, day);
	result.setSeconds(Math.round(seconds*1.318379));
	return result;
}

function jeolParamDate(aDate)
{
	return new Date(aDate*1000 + 631180800*1000); 
}

function brukerPulseWidth(aPw, aPulseSeq)
{
	var pw =  aPw*1.0e-6;
	if(aPulseSeq.indexOf("bp",0) != -1)
		pw = pw*2;
	
	return pw;
}

function brukerExpName(aFilePath, aExpName)
{
	if (aExpName != undefined && aExpName.length)
		return aExpName;
	else
	{
		var newPath = aFilePath.replace( "\\", "/");
		var pathArray = newPath.split("/");
		if( pathArray.length < 3 )
			return aFilePath;
		return pathArray[pathArray.length-3];
	}
}

function varianExpName(aFilePath)
{
	var newPath = aFilePath.replace( "\\", "/");
	var pathArray = newPath.split("/");
	if( pathArray.length < 2 )
		return aFilePath;
	var result = pathArray[pathArray.length-2];
	if (result.length > 4 && result.substring(result.length-4, result.length).toLowerCase() == ".fid")
		result = result.substring(0, result.length-4);
	return result;
}

function jcampDate(aJCAMP, aBruker, aVarian, aJeol)
{
	if (aJCAMP.length)
		return aJCAMP;
	if (aVarian.length)
		return varianDate(unBracket(aVarian, "<", ">"));
	if (aBruker.length)
		return brukerDate(aBruker);
	if (aJeol.length)
		return aJeol;
}

function jcampAcquiredSize(aBruker, aVarian, aJeol)
{
	if (aBruker.length)
		return div(aBruker, 2.0);
	if (aVarian.length)
		return div(aVarian, 2.0);
	if (aJeol.length)
		return aJeol;
	return "";
}

function brukerAQ(aAQ, aSW, aTD)
{
	if (!isNaN(parseFloat(aAQ)))
		return aAQ;
	else
		return 0.5*aTD/aSW;
}

function jcampAQ(aAQ, aSW, aTD)
{
	if (!isNaN(parseFloat(aAQ)))
		return aAQ;
	else
		return 0.5*aTD/aSW;
}

function jeolInstrument(aInstrument, aCode)
{
	if (aInstrument.length)
		return aInstrument;

	switch (parseInt(aCode, 10))
	{
		case 0: return "NONE";
		case 1: return "GSX";
		case 2: return "ALPHA";
		case 3: return "ECLIPSE";
		case 4: return "MASS_SPEC";
		case 5: return "COMPILER";
		case 6: return "OTHER_NMR";
		case 7: return "UNKNOWN";
		case 8: return "GEMINI";
		case 9: return "UNITY";
		case 10: return "ASPECT";
		case 11: return "UX";
		case 12: return "FELIX";
		case 13: return "LAMBDA";
		case 14: return "GE_1280";
		case 15: return "GE_OMEGA";
		case 16: return "CHEMAGNETICS";
		case 17: return "CDFF";
		case 18: return "GALACTIC";
		case 19: return "TRIAD";
		case 20: return "GENERIC_NMR";
		case 21: return "GAMMA";
		case 22: return "JCAMP_DX";
		case 23: return "AMX";
		case 24: return "DMX";
		case 25: return "ECA";
		case 26: return "ALICE";
		case 27: return "NMR_PIPE";
		default: return "";
	}
}

function calculateK(aDiffList)
{
	//autodetect units
	if(!aDiffList.length)
		return 1.0;
		
	var arrayStr = aDiffList.split("\n");
	var values = new Array();
	for( var i = 0; i < arrayStr.length; i++ ){
		var val = parseFloat(arrayStr[i]);
		if(!isNaN(val))
			values.push(val);
	}

	with(Math){
		values.sort( function(a, b){ return a-b;} ); //ascending order

		var maxG= values[values.length-1];
		var logMaxG = log(maxG)/LN10;		//log10(x) = ln(x)/ln(10)
		var dec = round(logMaxG);
		var k = pow(10.0, -(dec-2) );
	}

	return k;
}
