/******************************************************************************************************
Copyright (C) 2009 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/ 

// This function defines JACS-like peak report.
// To customize report, edit this function or implement another one.
function JACSPeakReporter()
{
	PeakReporter.call(this);
	
	this.name = new String("JACS");
	
// Define font size and font family
	this.font = "<font style=\"font-size: 10pt; font-family: Times New Roman\">"; 
		
	this.nucleusTemplate="%1";
// Report header. %1 will be raplaced with nucleusString, %2  with frequency, %3 with solvent 
	this.header = "%1 NMR (%2 MHz, %3) &delta; ";
//Report f1 value in 2D spectrum		
	this.reportf1In2D=false;
//Peak templates:
	this.peakTemplate="%1";//%1 - delta
	this.peakTemplate2D="(%2 %1)"; //%1 - delta f1, %2 - delta f2
		
	this.deltaPrecision = 2; // delta precision
	this.pSeparator = ", "; // peak separator 
		
	this.start = this.font;
	this.end = ".</font>";
}

JACSPeakReporter.prototype = new PeakReporter();
JACSPeakReporter.prototype.toString = function() { return "JACSPeakReporter()"; }

// This function defines Angewandte-like peak report.
// To customize report, edit this function or implement another one.
function AngewandtePeakReporter()
{
	PeakReporter.call(this);
	
	this.name = new String("Angewandte");
	
	this.onlyElementName=false;
// Define font size and font family
	this.font = "<font style=\"font-size: 10pt; font-family: Times New Roman\">"; 
		
	this.nucleusTemplate="%1";
// Report header. %1 will be raplaced with nucleusString, %2  with frequency, %3 with solvent 
	this.header = "%1 NMR (%2 MHz, %3) &delta; = ";
		//Report f1 value in 2D spectrum		
	this.reportf1In2D=false;
		
		//Peak templates:
	this.peakTemplate="%1";//%1 - delta
	this.peakTemplate2D="(%2 %1)"; //%1 - delta f1, %2 - delta f2
		
	this.deltaPrecision = 2; // delta precision
	this.pSeparator = ", "; // peak separator 
		
	this.start = this.font;
	this.end = ".</font>";
}

AngewandtePeakReporter.prototype = new PeakReporter();
AngewandtePeakReporter.prototype.toString = function() { return "AngewandtePeakReporter()"; }

// This function defines RSC-like peak report.
// To customize report, edit this function or implement another one.
function RSCPeakReporter()
{
	PeakReporter.call(this);
	
	this.name = new String("RSC");
	
	this.onlyElementName=true;
// Define font size and font family
	this.font = "<font style=\"font-size: 10pt; font-family: Times New Roman\">"; 

//%1 will be replaced with nucleus
	this.nucleusTemplate="<sub>%1</sub>";
// Report header. %1 will be raplaced with the value of nucleusTemplate, %2  with frequency, %3 with solvent 
	this.header = " &delta; %1 (%2 MHz, %3) ";
	//Report f1 value in 2D spectrum		
	this.reportf1In2D=false;
	
	//Peak templates:
	this.peakTemplate="%1";//%1 - delta
	this.peakTemplate2D="(%2 %1)"; //%1 - delta f1, %2 - delta f2
	
	this.deltaPrecision = 2; // delta precision
	this.pSeparator = ", "; // peak separator 
	
	this.start = this.font;
	this.end = ".</font>";
}

RSCPeakReporter.prototype = new PeakReporter();
RSCPeakReporter.prototype.toString = function() { return "RSCPeakReporter()"; }

// Warning! Do not delete the below function, do not rename it, it is called from MNova!
// Changing its contents is allowed. In particular, edit the type of peak reporter
function automaticPeakReport()
{
	return reportPeaksUser(false);
}

// Warning! Do not delete the below function, do not rename it! 
// Changing its contents is allowed.
function reportPeaksUser(createItem)
{
	if( Env.ReportPeaksUser != undefined )
		return Env.ReportPeaksUser(createItem);
	else
		return PeakReporter.reportPeaksUser(createItem);
}

function customizeReportPeaksUser()
{
	if( Env.ReportPeaksUserSetup != undefined )
		return Env.ReportPeaksUserSetup();
	else
	{
		if( Env.DefaultPeakReporter != undefined )
			MessageBox.information ( "Using Default Peak Reporter: "+ Env.DefaultPeakReporter.toString(), MessageBox.Ok, MessageBox.NoButton, MessageBox.NoButton, "Information" );
		else
		{
			var diag = new PeakReportDialog(Env.PeakReportersArray, true);
			diag.exec();
		}
	}
}

// This function defines peak list report.
// To customize report, edit this function or implement another one.
function PeakReporter()
{
	this.name = new String("Custom");
	// fields to be filled in
	this.font = new String;
	this.header = new String; 
	this.nucleusTemplate = new String;
	this.onlyElementName = false;
	this.peakTemplate = new String;
	this.peakTemplate2D = new String;
	this.reportf1In2D = false;
	this.deltaPrecision = 2; 
	this.pSeparator = new String;
	this.start = new String;
	this.end = new String;
}

PeakReporter.prototype.toString = function() { return "PeakReporter()"; }
PeakReporter.templateSettings = new String("Peak Report/Template");

PeakReporter.reportPeaksUser = function(createItem)
{
	var peakTable = new PeakTable(nmr.peakTable()); // Get table of peaks
	var spectrum;
	if(peakTable.isValid())
		spectrum = new NMRSpectrum(peakTable.spectrum()); // Get spectrum currently shown in the table of multiplets
	else 
		spectrum = new NMRSpectrum(nmr.activeSpectrum());
	
	var reporter = PeakReporter.getDefaultReporter("JACS");
	if( reporter != undefined )
		return reporter.report(spectrum, createItem);
}

PeakReporter.getReporterByName = function(aName)
{
	for( var i = 0; i < Env.PeakReportersArray.length; i++ )
	{
		if( Env.PeakReportersArray[i].name == aName )
			return Env.PeakReportersArray[i];
	}
	return undefined;
}

PeakReporter.getDefaultReporter = function(aDefValue)
{
	if( Env.DefaultPeakReporter != undefined )
	{
		var retVal = PeakReporter.getReporterByName(Env.DefaultPeakReporter);
		if( retVal != undefined )
			return retVal;
		else
			return PeakReporter.getReporterByName(aDefValue);
	}
	else
	{
		var defaultName = settings.value( PeakReporter.templateSettings, aDefValue );
		var retVal = PeakReporter.getReporterByName(defaultName);
		if( retVal != undefined )
			return retVal;
		else
			return PeakReporter.getReporterByName(aDefValue);
	}
}

PeakReporter.prototype.report = function (spectrum, createItem)
{
	var peaks = new Peaks(spectrum.peaks()); // get peaks from spectrum
	var dim = spectrum.dimCount;
	if( dim == 1 )
		peaks.sort(false);
	else
		peaks.sort(false,true);
	    
	const count = peaks.count;
	if (!count) // if no peaks then exit
		return "";
	    // Generate HTML report string
	var nucleusString = new String();
	if( this.onlyElementName )
		nucleusString = removeNumbers(spectrum.nucleus());
	else
		nucleusString=spectrum.nucleus(true);
	nucleusString = this.nucleusTemplate.replace("%1", nucleusString);
	var result = "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><title>Peak Report</title></head><body>" + this.start + this.header.replace("%1", nucleusString).replace("%2", Math.round(spectrum.frequency())).replace("%3", solventStr2Html(spectrum.solvent));
	    
	for(var i = 0; i < count; i++)
	{
		var peak = new Peak( peaks.at(i) );
		var pString = new String();
		if( dim == 1 )
			pString = this.peakTemplate.replace("%1", peak.delta(1).toFixed(this.deltaPrecision));
		else if( this.reportf1In2D )
			pString = this.peakTemplate2D.replace("%1", peak.delta(1).toFixed(this.deltaPrecision)).replace("%2", peak.delta(2).toFixed(this.deltaPrecision));
		else
			pString = this.peakTemplate.replace("%1", peak.delta(2).toFixed(this.deltaPrecision));
		result += pString;
		if(i<count-1)
			result += this.pSeparator;
	}
	result += this.end + "</body></html>";
	if( createItem )
	{
		var oldLTCount = mainWindow.activeWindow().itemCount("Layout Template");
		var textItem = draw.text(result, "Report Special", "Peaks,"+spectrum.nucleus(), true);
		if (oldLTCount == mainWindow.activeWindow().itemCount("Layout Template"))
		{
			textItem.left = spectrum.left;
			textItem.right = (spectrum.left+spectrum.right)/2.0;
			textItem.top = spectrum.top;
			textItem.bottom = spectrum.top + (spectrum.bottom-spectrum.top)/4.0;
			mainWindow.activeWindow().update();
		}
	}
	return result;
}

function reportPeaks( reporter )
{
	var spectrum = new NMRSpectrum(nmr.activeSpectrum());
	reporter.report(spectrum, true);
}


Env.PeakReportersArray.push( new JACSPeakReporter() );
Env.PeakReportersArray.push( new RSCPeakReporter() );
Env.PeakReportersArray.push( new AngewandtePeakReporter() );
