/******************************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the Mnova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

/*globals Str, Molecule, Application, NMRAssignments, NMRSpectrum, settings, print, MessageBox, MnUi*/
/*jslint plusplus: true, indent: 4*/

function AssignmentReporter(aArray, aDescription, aCorrelationDesc) {
	'use strict';
	this.fCorrelations = aArray;
	this.fDescription = aDescription;
	this.fCorrelationsDescription = aCorrelationDesc;
}

AssignmentReporter.prototype = {};

AssignmentReporter.assignmentReportCorrelations = function (decimals, aAssignmentObject, aMolecule, aAssignmentReporter, aMulti) {
	'use strict';
	var i, at, noEqHs, hIndex, atomLabel, atomRow, h, shift, hIsHeavyIndex, skip, shiftH, shiftH0, shiftH1, element,
		aCount = aMolecule.atomCount,
		tableRows = {},
		headerRow = [];

	if (aAssignmentReporter !== undefined) {
		for (i = 0; i < aAssignmentReporter.fCorrelations.length; i++) {
			headerRow.push(aAssignmentReporter.fCorrelationsDescription[i]);
		}
	}
	tableRows.header = headerRow;

	for (at = 1; at <= aCount; at++) {
		noEqHs = aAssignmentObject.notEqHs(at);
		skip = true;
		hIsHeavyIndex = false;
		atomLabel = aMolecule.atom(at).number;
		element = aMolecule.atom(at).elementSymbol;

		if (noEqHs.length === 0  && element === "C") {
			atomRow = [];
			atomRow[0] = AssignmentReporter.atomIndexToString(atomLabel, at);
			atomRow[1] = "";
			atomRow[2] = AssignmentReporter.findMultiplicity(decimals, aMulti, atomRow[1]);
			shift =  aAssignmentObject.chemShiftArr(at);
			if (shift) {
				if (shift[1]) {
					shiftH0 = Number((shift[0].max + shift[0].min) / 2).toFixed(decimals);
					shiftH1 = Number((shift[1].max + shift[1].min) / 2).toFixed(decimals);
					atomRow[3] = shiftH0 + "," + shiftH1;
				} else {
					atomRow[3] = Number((shift.max + shift.min) / 2).toFixed(decimals);
				}
			} else {
				atomRow[3] = "";
			}
			tableRows[atomRow[0]] = atomRow;
		} else {

			for (hIndex = 0; hIndex < noEqHs.length; hIndex++) {
				atomRow = [];
				atomRow[0] = AssignmentReporter.atomIndexToString(atomLabel, at);
				atomRow[1] = "";
				atomRow[2] = "";
				h = noEqHs[hIndex];
				if (h === 0) {
					hIsHeavyIndex = true;//H not attached to any C
				}
				shift =  aAssignmentObject.chemShiftArr(at, h);

				if (shift) {
					if (noEqHs.length > 1) {
						atomRow[0] = AssignmentReporter.atomIndexToString(atomLabel, at, h, true);
					} else if (noEqHs.length > 0) {
						atomRow[0] = AssignmentReporter.atomIndexToString(atomLabel, at, h, false);
					}
					skip = false;

					if (shift[1]) {
						shiftH0 = Number((shift[0].max + shift[0].min) / 2).toFixed(decimals);
						shiftH1 = Number((shift[1].max + shift[1].min) / 2).toFixed(decimals);
						atomRow[1] = shiftH0 + "," + shiftH1;

					} else {
						shiftH = Number((shift.max + shift.min) / 2).toFixed(decimals);
						if (atomRow[1] !== "") {
							shiftH = "," + shiftH;
							atomRow[1] += shiftH;
						} else {
							atomRow[1] = shiftH;
						}
					}
				}
				atomRow[2] = AssignmentReporter.findMultiplicity(decimals, aMulti, atomRow[1]);
				shift =  aAssignmentObject.chemShiftArr(at);
				if (!hIsHeavyIndex && shift && element === "C") {
					skip = false;

					if (shift[1]) {
						shiftH0 = Number((shift[0].max + shift[0].min) / 2).toFixed(decimals);
						shiftH1 = Number((shift[1].max + shift[1].min) / 2).toFixed(decimals);
						atomRow[3] = shiftH0 + "," + shiftH1;
					} else {
						atomRow[3] = Number((shift.max + shift.min) / 2).toFixed(decimals);
					}
				} else {
					atomRow[3] = "";
				}

				tableRows[atomRow[0]] = atomRow;
			}
		}
	}
	return tableRows;
};

AssignmentReporter.assignmentReportCorrelations2D = function (decimals, aAssignmentObject, aMolecule, aAssignmentReporter, aShowShift) {
	'use strict';
	var i, at, noEqHs, hIndex, atomRow, h, c, shift, atomLabel, element, correlations, correlationsObject, index, correlationsArray = [],
		aCount = aMolecule.atomCount,
		tableRows = {},
		headerRow = [];

	function sortFunctionForIntegers(a, b) {
		return a - b;
	}

	function sortFunctionForFloats(a, b) {
		return parseFloat(a) - parseFloat(b);
	}

	if (aAssignmentReporter !== undefined) {
		for (i = 0; i < aAssignmentReporter.fCorrelations.length; i++) {
			headerRow.push(aAssignmentReporter.fCorrelationsDescription[i]);
		}
	}
	tableRows.header = headerRow;

	for (at = 1; at <= aCount; at++) {
		noEqHs = aAssignmentObject.notEqHs(at);
		atomLabel = aMolecule.atom(at).number;
		element = aMolecule.atom(at).elementSymbol;

		for (hIndex = 0; hIndex < noEqHs.length; hIndex++) {
			atomRow = [];
			atomRow.push("");
			h = noEqHs[hIndex];
			shift =  aAssignmentObject.chemShiftArr(at, h);

			if (aAssignmentReporter !== undefined) {
				for (c = 0; c < aAssignmentReporter.fCorrelations.length; c++) {

					correlations = AssignmentReporter.correlationToString(aAssignmentObject, aMolecule, decimals, at, h, aAssignmentReporter.fCorrelations[c], aShowShift);

					if (!aShowShift && correlations !== "") {
						correlationsArray = correlations.split(",");

						correlationsObject = {};
						for (i = 0; i < correlationsArray.length; i++) {
							index = parseInt(correlationsArray[i], 10);
							if (!isNaN(index)) {
								correlationsObject[index] = 0;
							}
						}
						correlationsArray = [];
						for (i in correlationsObject) {
							if (correlationsObject.hasOwnProperty(i)) {
								correlationsArray.push(i);
							}
						}
						correlationsArray.sort(sortFunctionForIntegers);
						correlations = correlationsArray.toString();
					} else if (correlations !== "") {

						correlationsArray = correlations.split(",");
						correlationsArray.sort(sortFunctionForFloats);
						correlations = correlationsArray.toString();
					}

					atomRow.push(correlations);

					if (shift) {
						if (noEqHs.length > 1) {
							atomRow[0] = AssignmentReporter.atomIndexToString(atomLabel, at, h, true);
						} else if (noEqHs.length > 0) {
							atomRow[0] = AssignmentReporter.atomIndexToString(atomLabel, at, h, false);
						}
					}
				}
				tableRows[atomRow[0]] = atomRow;
			}
		}
	}
	return tableRows;
};

AssignmentReporter.getOrderedTable =  function (tableRows) {
	'use strict';

	var i, k, a, b, temp,
		len = tableRows.length - 1;

	for (i = 1; i < len; i++) {
		for (k = 1; k < len; k++) {
			a = parseFloat(tableRows[k][1]);
			b = parseFloat(tableRows[k + 1][1]);
			if (isNaN(a)) {
				a = 0;
			}
			if (isNaN(b)) {
				b = 0;
			}
			if (a < b) {
				temp = tableRows[k + 1];
				tableRows[k + 1] = tableRows[k];
				tableRows[k] = temp;
			}
		}
	}
	return tableRows;
};

AssignmentReporter.getPpmArray =  function (table) {
	'use strict';
	var i,
		ppmArray = [];

	for (i = 1; i < table.length; i++) {
		ppmArray[i] = table[i][1];
	}
	return ppmArray;
};

AssignmentReporter.findMultiplicity = function (decimals, multi, shift) {
	'use strict';

	function greaterThan(a, b) {
		return b - a;
	}

	var js, jArray, j,
		i = 0,
		found = false,
		multiplicity = "";

	while (i < multi.count && !found) {
		if (multi.at(i).delta.toFixed(decimals) === shift) {
			found = true;
			multiplicity = multi.at(i).category.toLowerCase();
			if (multiplicity !== "m" && multiplicity !== "s") {
				js = multi.at(i).jList();
				multiplicity += ",<i>J</i>=";
				jArray = [];
				for (j = 0; j < js.length; j++) {
					jArray[j] = js.at(j).toFixed(decimals);
				}

				jArray.sort(greaterThan); //sort descending

				for (j = 0; j < jArray.length; j++) {
					if (j === 0) {
						multiplicity += jArray[j];
					} else {
						multiplicity += "," + jArray[j];
					}
				}

				multiplicity += " Hz";
			}
		} else {
			i++;
		}
	}
	return multiplicity;
};

AssignmentReporter.atomIndexToString = function (aAtomLabel, aAtomNumber, aHIndex, aUseHIndex) {
	'use strict';
	var str = aAtomLabel;

	if (str === "") {
		str = aAtomNumber.toString();
	}
	if (aUseHIndex) {
		str += Str.hIndexToLetter(aHIndex);
	}
	return str;
};

AssignmentReporter.correlationToString = function (assignObject, aMolecule, decimals,  aAtom, aH, aCorrelation, aShowShift) {
	'use strict';
	var i, cShift, cMeanShift, atomLabel,
		corrString = "",
		noEqAtoms,
		useHIndex,
		corrAtoms;

	corrAtoms = assignObject.correlatedAtoms(aAtom, aH, aCorrelation);

	if (corrAtoms.length) {
		for (i = 0; i < corrAtoms.length; i++) {
			cShift = assignObject.chemShift(corrAtoms[i].atom, corrAtoms[i].indexes[0]);
			if (cShift !== undefined) {
				if (i > 0) {
					corrString  += ", ";
				}
				if (aShowShift) {
					cMeanShift = (cShift.max + cShift.min) / 2;
					corrString += cMeanShift.toFixed(decimals);
				}

				//				if (corrAtoms[i].atom !== aAtom) {
				atomLabel = aMolecule.atom(corrAtoms[i].atom).number;

				noEqAtoms = assignObject.notEqHs(corrAtoms[i].atom);
				if (corrAtoms[i].indexes.length === 1) {

					useHIndex = (noEqAtoms.length > 1);
					if (aShowShift) {
						corrString += "(";
					}
					corrString += AssignmentReporter.atomIndexToString(atomLabel, corrAtoms[i].atom, corrAtoms[i].indexes[0], useHIndex);
					if (aShowShift) {
						corrString += ")";
					}
				} else {
					if (aShowShift) {
						corrString += "(";
					}
					corrString += atomLabel;

					if (aShowShift) {
						corrString += ")";
					}
				}
				//				}
			}
		}
	}
	return corrString;
};

/** Gets the active molecule.
Returns the active molecule or the only one in the document
*/
function getActiveMolecule(aDocWin, aMolPlugin) {
	'use strict';
	var mol = aMolPlugin.activeMolecule();
	if (mol.isValid()) {
		return mol;
	}

	if (aDocWin.itemCount("Molecule") === 1) {
		mol = new Molecule(aDocWin.item(0, "Molecule"));
		return mol;
	}
	return undefined;
}

function createHTMLReport(table) {
	"use strict";
	var i, j, output;

	output = "<font style=\"font-size: 8pt; font-family: Arial; color: black\">";
	output += "<html><head>";
	output += "<title>Correlations Table</title>";
	output += "</head><body>";
	output += '\n<table border="1" cellSpacing="0" cellPadding="4" width="100%">';
	output += '\n<tr style="background-color:silver">';

	for (i = 0; i < table[0].length; i++) {
		output += '<td><b>' + table[0][i] + '</b></td>';
	}
	output += '</tr>';

	for (i = 1; i < table.length; i++) {
		output += '<tr>';
		for (j = 0; j < table[i].length; j++) {
			output += '<td>' + table[i][j] + '</td>';
		}
		output += '</tr>';
	}
	output += '</table>';
	output += '</body></html>';

	return output;
}


function removeVoidColumns(table) {
	"use strict";

	var i, j, row,
		counter = [],
		newTable = [];

	for (i = 0; i < table[0].length; i++) {
		counter.push(0);
	}

	for (i = 1; i < table.length; i++) {
		for (j = 0; j < table[0].length; j++) {
			if (table[i][j] !== "") {
				counter[j]++;
			}
		}
	}

	for (i = 0; i < table.length; i++) {
		row = [];
		for (j = 0; j < counter.length; j++) {
			if (counter[j] !== 0) {
				row.push(table[i][j]);
			}
		}
		newTable.push(row);
	}
	return newTable;
}

function getFinalTable(firstTable, secondTable) {
	"use strict";

	var i, joinedTable = [], aux = [];

	if (secondTable) {
		joinedTable.push(firstTable.header.concat(secondTable.header));
		for (i = 0; i < secondTable.header.length; i++) {
			aux.push("");
		}
	} else {
		joinedTable.push(firstTable.header);
	}

	for (i in firstTable) {
		if (firstTable.hasOwnProperty(i) && i !== "header") {
			if (secondTable) {
				if (secondTable[i]) {
					joinedTable.push(firstTable[i].concat(secondTable[i].slice(1)));
				} else {
					joinedTable.push(firstTable[i].concat(aux));
				}
			} else {
				joinedTable.push(firstTable[i]);
			}
		}
	}

	return joinedTable;
}


function assignmentReport() {
	'use strict';
	var mol, assign, spectra, foundMulti, specIndex, spectrum, multi, correlations2DKey, clipBoardKey, standardReporter, correlationsReporter,
		diag, drawnItems, tableText, pageItem, i, standardTable, correlationsTable, orderKey, decimalsKey, showShiftKey,
		dw = Application.mainWindow.activeDocument;

	function check2DOption() {

		if (diag.widgets.ck2DCorrelations.checked) {
			diag.widgets.ckShowShift.enabled = true;
		} else {
			diag.widgets.ckShowShift.enabled = false;
		}
	}

	if (dw === undefined || Application.molecule === undefined) {
		return;
	}

	mol = getActiveMolecule(dw, Application.molecule);
	if (mol === undefined || !mol.isValid()) {
		MessageBox.critical("Invalid Molecule");
		return;
	}

	assign = new NMRAssignments(mol);
	spectra = dw.itemCount("NMR Spectrum");
	foundMulti = false;
	specIndex = 0;
	while (!foundMulti  && specIndex < spectra) {
		spectrum = new NMRSpectrum(dw.item(specIndex, "NMR Spectrum"));

		if ((spectrum.nucleus() === "1H") && (spectrum.dimCount === 1)) {
			foundMulti = true;
			multi = spectrum.multiplets();
		} else {
			specIndex++;
		}
	}
	if (!foundMulti) {
		MessageBox.critical("Invalid Spectrum");
		return;
	}
	correlationsReporter = new AssignmentReporter(["HSQC", "HMBC", "H2BC", "COSY", "NOESY", "TOCSY", "ROESY"], "2D Correlations", [assign.realAssignedExp("HSQC"), "HMBC", "H2BC", assign.realAssignedExp("COSY"), assign.realAssignedExp("NOESY"), assign.realAssignedExp("TOCSY"), "ROESY"], "Correlation Reporter/2D");
	standardReporter = new AssignmentReporter(["No", "&delta; <sub>H</sub>", "Mul", "&delta; <sub>C</sub>"], "Main", ["No", "&delta; <sub>H</sub>", "Multiplicity, <i>J</i> (Hz)", "&delta; <sub>C</sub>"], "Correlation Reporter/H&C");

	clipBoardKey = "Correlation Reporter/ClipBoard";
	correlations2DKey  = "Correlation Reporter/2D Correlations";
	orderKey = "Correlation Reporter/Order by shift";
	decimalsKey = "Correlation Reporter/Number of decimals";
	showShiftKey = "Correlation Reporter/Show Shift";

	diag = Application.loadUiFile("ricares:assignmentReport.ui");

	diag.widgets.ck2DCorrelations.checked = settings.value(correlations2DKey, true);
	diag.widgets.ckClipBoard.checked = settings.value(clipBoardKey, true);
	diag.widgets.sbDecimals.value = settings.value(decimalsKey, 2);
	diag.widgets.ckOrder.checked = settings.value(orderKey, true);
	diag.widgets.ckShowShift.checked = settings.value(showShiftKey, true);
	diag.widgets.ck2DCorrelations.toggled.connect(check2DOption);
	check2DOption();

	drawnItems = [];

	if (diag.exec()) {

		standardTable = AssignmentReporter.assignmentReportCorrelations(diag.widgets.sbDecimals.value, assign, mol, standardReporter, multi);

		if (diag.widgets.ck2DCorrelations.checked) {
			correlationsTable = AssignmentReporter.assignmentReportCorrelations2D(diag.widgets.sbDecimals.value, assign, mol, correlationsReporter, diag.widgets.ckShowShift.checked);
		}
		standardTable = getFinalTable(standardTable, correlationsTable);

		if (diag.widgets.ckOrder.checked) {
			standardTable = AssignmentReporter.getOrderedTable(standardTable);
		}

		tableText = createHTMLReport(removeVoidColumns(standardTable));
		pageItem = Application.draw.text(tableText, "Report Special", "Assignments Proton", true);
		drawnItems.push(pageItem);

		for (i = 1; i < drawnItems.length; i++) {
			drawnItems[i].top = drawnItems[i - 1].top;
			drawnItems[i].left = drawnItems[i - 1].right;
		}
		if (!diag.widgets.ckClipBoard.checked) {
			settings.setValue(clipBoardKey, false);
			Application.mainWindow.activeDocument.curPage().update();
		} else {
			settings.setValue(clipBoardKey, true);
			dw.setSelection(drawnItems);
			Application.mainWindow.doAction("action_Edit_Copy");
			Application.mainWindow.activeDocument.curPage().deleteItems(drawnItems);
			dw.update();
		}
		settings.setValue(correlations2DKey, diag.widgets.ck2DCorrelations.checked);
		settings.setValue(clipBoardKey, diag.widgets.ckClipBoard.checked);
		settings.setValue(decimalsKey, diag.widgets.sbDecimals.value);
		settings.setValue(orderKey, diag.widgets.ckOrder.checked);
		settings.setValue(showShiftKey, diag.widgets.ckShowShift.checked);
	}
}

if (this.MnUi && MnUi.scripts_nmr) {
	MnUi.scripts_nmr.scripts_nmr_ReportAssignments = assignmentReport;
}
