/*******************************************************************************************
Copyright (C) 2014 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*******************************************************************************************/

/*globals nmr, settings, Dir, FileDialog, File, TextStream, Application, MnUi*/
/*jslint plusplus: true, indent: 4*/

function exportDmfit() {
	"use strict";
	var fileName, fout, sout, hz, dHz, i, scale, fromPt, toPt, tmp, scaleUnits,
		spec = nmr.activeSpectrum(),
		dirSetKey = "exportDmfit/Dir",
		saveDir = settings.value(dirSetKey, Dir.home());
	if (!spec.isValid()) {
		return;
	}
	if (spec.dimCount !== 1) {
		return;
	}

	fileName = FileDialog.getSaveFileName("ASCII Files (*.txt)", "", saveDir);
	if (!fileName.length) {
		return;
	}

	fout = new File(fileName);
	try {
		settings.setValue(dirSetKey, fout.absDirPath);

		fout.open(File.WriteOnly);
		sout = new TextStream(fout);
		sout.precision = 10;

		sout.writeln("ti:", spec.title.replace(/\n|\n\r/g, " "));
		sout.writeln("##freq ", spec.frequency());

		scaleUnits = spec.getProperty("axes.horizontal.units");
		scale = spec.getScaleLimits();
		fromPt = Math.round(spec.unitsToUnits(scaleUnits, "pt", scale.fromX, 1));
		toPt = Math.round(spec.unitsToUnits(scaleUnits, "pt", scale.toX, 1));
		if (fromPt > toPt) {
			tmp = fromPt;
			fromPt = toPt;
			toPt = tmp;
		}

		hz = spec.hz() + spec.scaleWidth();
		dHz = spec.scaleWidth() / spec.count();
		hz -= fromPt * dHz;
		for (i = fromPt; i < toPt; i++) {
			sout.writeln(hz, "\t", spec.real(i));
			hz -= dHz;
		}
	} finally {
		fout.close();
	}
}

if (this.MnUi && MnUi.scripts_nmr) {
	MnUi.scripts_nmr.scripts_nmr_ExportASCIIDmfit = exportDmfit;
}
