/*******************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the Mnova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*******************************************************************************************/

/*globals NMRSpectrum, nmr, settings, Dir, FileDialog, File, TextStream, Application, MnUi*/
/*jslint plusplus: true, indent: 4*/

function exportFID() {
	"use strict";

	var spec = nmr.activeSpectrum(),
		dirSetKey,
		saveDir,
		fileName,
		fout,
		sout,
		dt,
		t,
		i,
		fcount;

	if (!spec.isValid()) {
		return;
	}
	if (spec.dimCount !== 1) {
		return;
	}

	dirSetKey = "exportFID/Dir";
	saveDir = settings.value(dirSetKey, Dir.home());
	fileName = FileDialog.getSaveFileName("ASCII Files (*.txt)", "", saveDir);
	if (fileName.length === 0) {
		return;
	}

	fout = new File(fileName);

	settings.setValue(dirSetKey, fout.absDirPath);

	fout.open(File.WriteOnly);
	sout = new TextStream(fout);
	sout.precision = 10;

	sout.writeln("Frequency\t", spec.frequency());
	sout.writeln("Nucleus\t", spec.nucleus());

	dt = 1.0 / spec.scaleWidth();
	t = 0.0;
	for (i = 0, fcount = spec.fidCount(); i < fcount; i++) {
		sout.writeln(t, "\t", spec.fidReal(i), "\t", spec.fidImag(i));
		t += dt;
	}
	fout.close();
}

if (this.MnUi && MnUi.scripts_nmr) {
	MnUi.scripts_nmr.scripts_nmr_ExportASCIIFID = exportFID;
}
