/*******************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*******************************************************************************************/

/*globals NMRSpectrum, NMRProcessing, nmr, settings, Dir, FileDialog, TextStream, File, Application, MnUi*/
/*jslint plusplus: true, indent: 4*/

function exportGDCorrectedFID() {
	"use strict";
	var fileName, fout, sout, p, i,
		dirSetKey = "exportGDCorrectedFID/Dir",
		saveDir = settings.value(dirSetKey, Dir.home()),
		spec = nmr.activeSpectrum();
	if (!spec.isValid()) {
		return;
	}

	fileName = FileDialog.getSaveFileName("ASCII Files (*.txt)", "", saveDir);
	if (!fileName) {
		return;
	}

	fout = new File(fileName);
	settings.setValue(dirSetKey, fout.absDirPath);

	fout.open(File.WriteOnly);
	try {
		sout = new TextStream(fout);
		sout.precision = 10;

		p = spec.proc;
		for (i = spec.dimCount; i > 0; i--) {
			sout.writeln("Size[" + i + "] = " + spec.count(i));
			sout.writeln("Nucleus[" + i + "] = " + spec.nucleus(i));
			sout.writeln("SpectrometerFrequency[" + i + "] = " + spec.frequency(i));
			sout.writeln("SpectralWidth[" + i + "] = " + spec.scaleWidth(i));
			sout.writeln("Hz[" + i + "] = " + spec.hz(i));
			sout.writeln("Zero[" + i + "] = " + spec.zero(i));
			sout.writeln("RealFT[" + i + "] = " + p.getParameter("FT[" + i + "].RealFT"));
			sout.writeln("Quadrature[" + i + "] = " + p.getParameter("FT[" + i + "].Quadrature"));
			sout.writeln("Invert[" + i + "] = " + p.getParameter("FT[" + i + "].Invert"));
			sout.writeln("Protocol[" + i + "] = " + p.getParameter("FT[" + i + "].Protocol"));
		}
		sout.writeln("DataPoints = ");
	} finally {
		fout.close();
	}
	spec.exportGDCorrectedFID(fileName);
}

if (this.MnUi && MnUi.scripts_nmr) {
	MnUi.scripts_nmr.scripts_nmr_ExportASCIIGroupDelayCorrectedFID = exportGDCorrectedFID;
}
