/******************************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

/*globals Env, draw, mainWindow*/
/*jslint plusplus: true, indent: 4*/

//the GUI will call this function with 3 parameters, the Graph object to be reported, and two booleans to create an item or not and to use html or not.
Env.DefaultGraphReport = function (aGraph, aCreateItem, aHtml) {
	'use strict';
	var sCount, s, fitModels, yModels, reportText, graphProperties, horzAxsLabel, yModelLabel, formula, parameters, template, parametersStr, p, parameterTemplate, textItem, htmlTemplate, oldLTCount, precision;

	if (aGraph === undefined || !aGraph.isValid()) {
		throw new Error("Invalid Graph");
	}

	precision = 4;
	htmlTemplate = "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><title>Graph Report</title></head><body>%1</body></html>"; //%1 html body
	if (aHtml) {
		template = "<p><b>%1' = %2</b><br/>%3<br/>Error: %4</p>"; //%1: label, %2: formula, %3: parameters, %4: comments
	} else {
		template = "%1' = %2\n%3\nError: %4\n";
	}
	parameterTemplate = '%1=%2;'; //%1: name, %2: value
	reportText = '';
	sCount = aGraph.seriesCount;
	fitModels = aGraph.fitModels;
	yModels = aGraph.yModels;
	graphProperties = aGraph.graphicProperties;
	horzAxsLabel = graphProperties.horzAxis.label;
	horzAxsLabel = horzAxsLabel.replace(/\([\w\W]*\)/, '');
	for (s = 0; s < sCount; ++s) {
		yModelLabel = yModels[s].label;
		if (yModelLabel.length === 0) {
			if (s === 0) {
				yModelLabel = 'Y';
			} else {
				yModelLabel = 'Y%1'.arg(s);
			}
		}
		formula = fitModels[s].formula;
		if (formula.length !== 0) {
			formula = formula.replace(/exp/g, 'EXP');
			formula = formula.replace(/x/g, horzAxsLabel);
			formula = formula.replace(/EXP/g, 'exp');
			parameters = fitModels[s].fitParams;
			parametersStr = '';
			for (p in parameters) {
				if (parameters.hasOwnProperty(p)) {
					if (parametersStr.length !== 0) {
						parametersStr += ' ';
					}
					parametersStr += parameterTemplate.arg(p).arg(parameters[p].toPrecision(precision));
				}
			}
			reportText += template.arg(yModelLabel).arg(formula).arg(parametersStr).arg(fitModels[s].error.toPrecision(precision));
		}
	}
	if (aHtml) {
		reportText = htmlTemplate.arg(reportText);
	}
	if (aCreateItem) {
		oldLTCount = mainWindow.activeDocument.itemCount("Layout Template");
		textItem = draw.text(reportText, "Report Special", "Graph", true);
		if (oldLTCount === mainWindow.activeDocument.itemCount("Layout Template")) {
			textItem.width = mainWindow.activeDocument.curPage().width - aGraph.width;
			textItem.height = aGraph.height;
			textItem.left = aGraph.left;
			textItem.top = aGraph.top;
			textItem.translate(aGraph.width, 0.0);
			textItem.update();
			mainWindow.activeDocument.update();
		}
	}
	return reportText;
};

