/*******************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the Mnova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*******************************************************************************************/

/*globals settings, FileDialog, serialization, Dir, File, BinaryStream, Str, TextStream, Application, NMRSpectrum, MnUi*/
/*jslint plusplus: true, indent: 4, bitwise: true*/

function importAcdlabsEsp() {
	"use strict";

	function acdNuc2Str(aNuc) {
		switch (aNuc) {
		case 1: return "1H";
		case 2: return "13C";
		}
	}

	var fi, fileName, fin, sin, len, signature, ftFlag, npts, freq, nuc, sw, sc, paramsOk, parId, size,
		txtFileName, fout, sout, reArr, i, dt, t, dPpm, ppm, oldItemsCount, newItemsCount, spec, proc,
		dirSetKey = "importAcdlabsEsp/Dir",
		openDir = settings.value(dirSetKey, Dir.home()),
		fileNames = FileDialog.getOpenFileNames("ACDLabs 1D NMR (*.esp)", "", openDir);
	if (!fileNames.length) {
		return;
	}
	serialization.fileHistory = false;
	for (fi = 0; fi < fileNames.length; fi++) {
		fileName = fileNames[fi];

		fin = new File(fileName);

		if (!fi) {
			settings.setValue(dirSetKey, fin.absDirPath);
		}

		fin.open(File.ReadOnly);
		sin = new BinaryStream(fin);
		sin.endianness = BinaryStream.eLittle;

		len = sin.readInt16();
		if (len !== 0x0C) {
			fin.close();
		} else {
			signature = Str.charCodeArrayToString(sin.readBytes(len));
			if (signature !== "(C) ACD 1994") {
				fin.close();
			} else {
				len = sin.readInt16();
				if (len !== 0x0E && len !== 0x10) {
					fin.close();
				} else {
					signature = Str.charCodeArrayToString(sin.readBytes(len));
					if (signature !== ".ESP.( V 1.0 )" && signature !== "D1EspVersionInfo") {
						fin.close();
					} else {
						if (signature === "D1EspVersionInfo") {
							len = sin.readInt16();
							sin.pos += len + 0x09;
						} else {
							sin.pos += 0x0B;
						}
						paramsOk = true;
						do {
							sin.pos -= 1;
							parId = sin.readInt8();
							size = sin.readInt8();
							switch (parId) {
							case 0x01:
								if (size !== 2) {
									paramsOk = false;
								}
								ftFlag = sin.readInt16();
								break;
							case 0x02:
								if (size !== 4) {
									paramsOk = false;
								}
								npts = sin.readInt32();
								break;
							case 0x03:
								if (size !== 4) {
									paramsOk = false;
								}
								sw = sin.readReal32();
								break;
							case 0x04:
								if (size !== 4) {
									paramsOk = false;
								}
								sc = sin.readReal32();
								break;
							case 0x05:
								if (size !== 4) {
									paramsOk = false;
								}
								freq = sin.readReal32();
								break;
							case 0x0C:
								if (size !== 1) {
									paramsOk = false;
								}
								nuc = sin.readInt8();
								break;
							default:
								sin.pos += size;
							}
						} while (paramsOk && sin.readInt8());

						if (!paramsOk || !npts) {
							fin.close();
						} else {
							if (sin.readInt8()) {
								fin.close();
							} else {
								txtFileName = Dir.temp() + "/" + Str.fileNameFromPath(fileName) + ".txt";
								fout = new File(txtFileName);
								fout.open(File.WriteOnly);
								sout = new TextStream(fout);
								sout.precision = 10;

								sout.writeln("Frequency\t", freq);
								sout.writeln("Nucleus\t", acdNuc2Str(nuc));
								if (ftFlag & 0x01) {
									sout.writeln("Domain\ttime");
								} else if (ftFlag & 0x02) {
									sout.writeln("Domain\tfrequency");
								}

								reArr = [];
								for (i = 0; i < npts; i++) {
									reArr[i] = sin.readReal32();
								}
								if (ftFlag & 0x01) {
									dt = 1 / sw;
									t = 0;
									for (i = 0; i < npts; i++) {
										sout.writeln(t, "\t", reArr[i], "\t", sin.readReal32());
										t += dt;
									}
								} else if (ftFlag & 0x02) {
									dPpm = sw / npts / freq;
									ppm = (sc - 0.5 * sw) / freq;
									for (i = 0; i < npts; i++) {
										sout.writeln(ppm, "\t", reArr[i], "\t", sin.readReal32());
										ppm += dPpm;
									}
								}

								fin.close();
								fout.close();

								oldItemsCount = (Application.mainWindow.activeDocument === undefined) ? 0 : Application.mainWindow.activeDocument.itemCount();
								serialization.open(txtFileName);
								newItemsCount = (Application.mainWindow.activeDocument === undefined) ? 0 : Application.mainWindow.activeDocument.itemCount();
								if (oldItemsCount + 1 === newItemsCount) {
									spec = new NMRSpectrum(Application.mainWindow.activeDocument.curPage().item(0));
									if (spec.isValid()) {
										proc = spec.proc;
										if (ftFlag & 0x01) {
											proc.setParameter("FT[1].Invert", true);
											proc.setParameter("FT[1].Quadrature", true);
										}
										proc.setParameter("PC.Method", "Global");
										spec.process(proc);
										spec.update();
									}
								}

								fout.remove();
							}
						}
					}
				}
			}
		}
	}
	if (Application.mainWindow.activeDocument !== undefined) {
		Application.mainWindow.activeDocument.update();
	}
	serialization.fileHistory = true;
}

if (this.MnUi && MnUi.scripts_nmr) {
	MnUi.scripts_nmr.scripts_nmr_ImportACDLabs1DNMR = importAcdlabsEsp;
}
