/*******************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the Mnova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*******************************************************************************************/

/*globals settings, Dir, FileDialog, serialization, File, TextStream, Str, Application, NMRSpectrum, MnUi*/
/*jslint plusplus: true, indent: 4*/

function importLCModelAscii() {
	"use strict";

	function getLCModelAsciiParam(aParamName, aHeader) {
		var eqPos, nlPos,
			parPos = aHeader.indexOf(aParamName);
		if (parPos === -1) {
			return undefined;
		}

		eqPos = aHeader.indexOf("=", parPos);
		if (eqPos === -1) {
			return undefined;
		}

		nlPos = aHeader.indexOf("\n", eqPos);
		if (nlPos === -1) {
			nlPos = aHeader.length;
		}

		return aHeader.substring(eqPos + 1, nlPos).replace(/^[\t ]*/, "");
	}

	var fi, fileName, fin, sin, header, headerEnd, dataStr, data, txtFileName, fout, sout,
		npts, t, dt, i, oldItemsCount, newItemsCount, spec, proc,
		dirSetKey = "importLCModelAscii/Dir",
		openDir = settings.value(dirSetKey, Dir.home()),
		fileNames = FileDialog.getOpenFileNames("LCModel ASCII (*.raw)", "", openDir);
	if (!fileNames.length) {
		return;
	}
	serialization.fileHistory = false;
	for (fi = 0; fi < fileNames.length; fi++) {
		fileName = fileNames[fi];

		fin = new File(fileName);

		if (!fi) {
			settings.setValue(dirSetKey, fin.absDirPath);
		}

		fin.open(File.ReadOnly);
		sin = new TextStream(fin);

		header = sin.readAll();

		headerEnd = header.lastIndexOf("$END\n");
		if (headerEnd === -1) {
			fin.close();
		} else {
			dataStr = header.substring(headerEnd + 6, header.length);
			dataStr = dataStr.replace(/[ \t\n\r]*$/, "");
			data = dataStr.split("\n");
			header = header.substring(0, headerEnd + 5);

			txtFileName = Dir.temp() + "/" + Str.fileNameFromPath(fileName) + ".txt";
			fout = new File(txtFileName);
			fout.open(File.WriteOnly);
			sout = new TextStream(fout);
			sout.precision = 10;

			sout.writeln("Frequency\t", getLCModelAsciiParam("hzpppm", header));
			sout.write(header);

			npts = data.length;
			t = 0.0;
			dt = getLCModelAsciiParam("deltat", header);
			if (!dt) {
				dt = 0.0005;
			}
			for (i = 0; i < npts; i++) {
				sout.writeln(t, "\t", data[i]);
				t += dt;
			}

			fin.close();
			fout.close();

			oldItemsCount = (Application.mainWindow.activeDocument === undefined) ? 0 : Application.mainWindow.activeDocument.itemCount();
			serialization.open(txtFileName);
			newItemsCount = (Application.mainWindow.activeDocument === undefined) ? 0 : Application.mainWindow.activeDocument.itemCount();
			if (oldItemsCount + 1 === newItemsCount) {
				spec = new NMRSpectrum(Application.mainWindow.activeDocument.curPage().item(0));
				if (spec.isValid()) {
					proc = spec.proc;
					proc.setParameter("FT[1].Invert", true);
					proc.setParameter("FT[1].Quadrature", true);
					spec.process(proc);
					spec.update();
				}
			}

			fout.remove();
		}
	}
	if (Application.mainWindow.activeDocument !== undefined) {
		Application.mainWindow.activeDocument.update();
	}
	serialization.fileHistory = true;
}

if (this.MnUi && MnUi.scripts_nmr) {
	MnUi.scripts_nmr.scripts_nmr_ImportLCModelASCII = importLCModelAscii;
}
