/*******************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the Mnova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*******************************************************************************************/

/*globals settings, Dir, FileDialog, serialization, print, File, TextStream, BinaryStream, Application, NMRSpectrum, Str, MnUi*/
/*jslint plusplus: true, bitwise: true, indent: 4*/

function vaxBytesToFloat32(aBytes) {
	"use strict";
	var b1 = aBytes[1],
		b2 = aBytes[0],
		b3 = aBytes[3],
		b4 = aBytes[2],

		S = (b1 & 0x80) >> 7,
		E = ((b1 & 0x7f) << 1) + ((b2 & 0x80) >> 7),
		F = ((b2 & 0x7f) << 16) + (b3 << 8) + b4,

		M = 0.5 + F / 16777216.0;

	if (E > 0) {
		return (S ? -1 : 1) * M * Math.pow(2.0, E - 128);
	} else if (E === 0 && S === 0) {
		return 0.0;
	} else if (E === 0 && S === 1) {
		return -1.0;
	}

	return 0.0;
}

function importPhilipsSdat() {
	"use strict";

	function getPhilipsSpar(aParamName, aParams) {
		var nlPos,
			parPos = aParams.indexOf(aParamName);
		if (parPos === -1) {
			return undefined;
		}

		nlPos = aParams.indexOf("\n", parPos);
		if (nlPos === -1) {
			nlPos = aParams.length;
		}

		return aParams.substring(parPos + aParamName.length + 2, nlPos).replace(/^ */, "");
	}

	var fi, sdatName, sparName, fspar, fsdat, sspar, params, ssdat, rows, npts, dt, r, txtFileName, fout, sout, t, i, re, im,
		oldItemsCount, newItemsCount, spec, proc,
		dirSetKey = "importPhilipsSdat/Dir",
		openDir = settings.value(dirSetKey, Dir.home()),
		fileNames = FileDialog.getOpenFileNames("Philips Achieva (*.SDAT)", "", openDir);

	if (!fileNames.length) {
		return;
	}
	serialization.fileHistory = false;
	for (fi = 0; fi < fileNames.length; fi++) {
		sdatName = fileNames[fi];
		sparName = sdatName.substring(0, sdatName.lastIndexOf(".")) + ".SPAR";
		print(sparName);

		if (File.exists(sparName)) {
			fspar = new File(sparName);
			fspar.open(File.ReadOnly);
			fsdat = new File(sdatName);
			fsdat.open(File.ReadOnly);

			if (!fi) {
				settings.setValue(dirSetKey, fspar.absDirPath);
			}

			sspar = new TextStream(fspar);
			params = sspar.readAll();
			fspar.close();

			ssdat = new BinaryStream(fsdat);
			ssdat.endianness = BinaryStream.eLittle;

			rows = Number(getPhilipsSpar("rows", params));
			npts = Number(getPhilipsSpar("dim1_pnts", params));
			dt = Number(getPhilipsSpar("dim1_step", params));
			for (r = 0; r < rows; r++) {
				txtFileName = Dir.temp() + "/" + Str.fileNameFromPath(sdatName) + ".txt";
				fout = new File(txtFileName);
				fout.open(File.WriteOnly);
				sout = new TextStream(fout);
				sout.precision = 10;

				sout.writeln("Frequency\t", getPhilipsSpar("synthesizer_frequency", params) / 1e6);
				sout.writeln("Nucleus\t", getPhilipsSpar("nucleus", params));
				sout.writeln("Domain\ttime");
				sout.writeln(params);

				t = 0.0;
				for (i = 0; i < npts; i++) {
					re = vaxBytesToFloat32(ssdat.readBytes(4));
					im = vaxBytesToFloat32(ssdat.readBytes(4));
					sout.writeln(t, "\t", re, "\t", im);
					t += dt;
				}

				fout.close();

				oldItemsCount = (Application.mainWindow.activeDocument === undefined) ? 0 : Application.mainWindow.activeDocument.itemCount();
				serialization.open(txtFileName);
				newItemsCount = (Application.mainWindow.activeDocument === undefined) ? 0 : Application.mainWindow.activeDocument.itemCount();
				if (oldItemsCount + 1 === newItemsCount) {
					spec = new NMRSpectrum(Application.mainWindow.activeDocument.curPage().item(0));
					if (spec.isValid()) {
						proc = spec.proc;
						proc.setParameter("FT[1].Quadrature", true);
						proc.setParameter("PC.Method", "Global");
						spec.process(proc);
						spec.update();
					}
				}

				fout.remove();
			}
		}
		fsdat.close();
	}
	if (Application.mainWindow.activeDocument !== undefined) {
		Application.mainWindow.activeDocument.update();
	}
	serialization.fileHistory = true;
}

if (this.MnUi && MnUi.scripts_nmr) {
	MnUi.scripts_nmr.scripts_nmr_ImportPhilipsAchieva = importPhilipsSdat;
}
