/*******************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*******************************************************************************************/

/*globals settings, Dir, FileDialog, serialization, File, BinaryStream, TextStream, Str, Application, NMRSpectrum, MnUi*/
/*jslint plusplus: true, indent: 4*/

function importSiemensRda() {
	"use strict";

	function getSiemensRdaParam(aParamName, aHeader) {
		var nlPos,
			parPos = aHeader.indexOf(aParamName);
		if (parPos === -1) {
			return undefined;
		}

		nlPos = aHeader.indexOf("\n", parPos);
		if (nlPos === -1) {
			nlPos = aHeader.length;
		}

		return aHeader.substring(parPos + aParamName.length + 1, nlPos).replace(/^ */, "");
	}

	var fi, fileName, fin, sin, header, headerEnd, txtFileName, fout, sout,
		npts, t, dt, i, oldItemsCount, newItemsCount, spec, proc,
		dirSetKey = "importSiemensRda/Dir",
		openDir = settings.value(dirSetKey, Dir.home()),
		fileNames = FileDialog.getOpenFileNames("Siemens Syngo (*.rda)", "", openDir);

	if (!fileNames.length) {
		return;
	}
	serialization.fileHistory = false;
	for (fi = 0; fi < fileNames.length; fi++) {
		fileName = fileNames[fi];
		fin = new File(fileName);

		if (!fi) {
			settings.setValue(dirSetKey, fin.absDirPath);
		}

		fin.open(File.ReadOnly);
		sin = new BinaryStream(fin);
		sin.endianness = BinaryStream.eLittle;

		header = Str.charCodeArrayToString(sin.readBytes(fin.size));

		headerEnd = header.indexOf(">>> End of header <<<");
		if (headerEnd === -1) {
			fin.close();
		} else {
			header = header.substring(0, headerEnd + 21);

			sin.pos = headerEnd + 23;

			txtFileName = Dir.temp() + "/" + Str.fileNameFromPath(fileName) + ".txt";
			fout = new File(txtFileName);
			fout.open(File.WriteOnly);
			sout = new TextStream(fout);
			sout.precision = 10;

			sout.writeln("Frequency\t", getSiemensRdaParam("MRFrequency", header));
			sout.writeln("Nucleus\t", getSiemensRdaParam("Nucleus", header));
			sout.writeln("Domain\ttime");
			sout.writeln(header);

			npts = Number(getSiemensRdaParam("VectorSize", header));
			t = 0.0;
			dt = getSiemensRdaParam("DwellTime", header) * 1e-6;
			for (i = 0; i < npts; i++) {
				sout.writeln(t, "\t", sin.readReal64(), "\t", sin.readReal64());
				t += dt;
			}

			fin.close();
			fout.close();

			oldItemsCount = (Application.mainWindow.activeDocument === undefined) ? 0 : Application.mainWindow.activeDocument.itemCount();
			serialization.open(txtFileName);
			newItemsCount = (Application.mainWindow.activeDocument === undefined) ? 0 : Application.mainWindow.activeDocument.itemCount();
			if (oldItemsCount + 1 === newItemsCount) {
				spec = new NMRSpectrum(Application.mainWindow.activeDocument.curPage().item(0));
				if (spec.isValid()) {
					proc = spec.proc;
					proc.setParameter("FT[1].Invert", true);
					proc.setParameter("FT[1].Quadrature", true);
					spec.process(proc);
					spec.update();
				}
			}

			fout.remove();
		}
	}
	if (Application.mainWindow.activeDocument !== undefined) {
		Application.mainWindow.activeDocument.update();
	}
	serialization.fileHistory = true;
}

if (this.MnUi && MnUi.scripts_nmr) {
	MnUi.scripts_nmr.scripts_nmr_ImportSiemensSyngo = importSiemensRda;
}
