//<GUI menuname="Multi-Open..." tooltip="Open Multiple Files"/>
/******************************************************************************************************
Copyright (C) 2008 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

function onOpenDir()
{
	var dirName = this.text;
	dirName = FileDialog.getExistingDirectory(dirName);
	print("Directory: " + dirName);
	this.text = dirName;
}

function multiOpen()
{
	const dirSettingsKey = "Multi Open/Last Directory";
	const fileMaskKey = "Multi Open/File Mask";
	const optFilesKey = "Multi Open/Optional Files";

	var dirName = settings.value(dirSettingsKey, Dir.home());
	var dirName2 = settings.value(dirSettingsKey+"2", Dir.home());

	var dialog = new Dialog();
	dialog.title = "Multi-Open";
	
	var gbMain = new GroupBox();
	gbMain.title = "Main Files";
	dialog.add(gbMain);

	var leDirectory = new LineEdit;
	leDirectory.label = "Base Directory:";
	leDirectory.text = dirName;

	var tbOpenDir = new ToolButton();
	tbOpenDir.text = "...";
	tbOpenDir.clicked.connect(leDirectory, onOpenDir); 
	gbMain.add(leDirectory, tbOpenDir);

	var leFileMask = new LineEdit;
	leFileMask.label = "Path Mask:";
	leFileMask.text = settings.value(fileMaskKey, "");
	leFileMask.toolTip = "Examples:\nPRJ ??????/\*.fid/*.mol\n\SMPL-*/10/pdata/1r";
	gbMain.add(leFileMask);	
	
	var gbOpt = new GroupBox();
	gbOpt.title = "Optional Files";
	gbOpt.checkable = true;
	gbOpt.checked = settings.value(optFilesKey, false);
	dialog.add(gbOpt);

	var leDirectory2 = new LineEdit;
	leDirectory2.label = "Base Directory:";
	leDirectory2.text = dirName2;

	var tbOpenDir2 = new ToolButton();
	tbOpenDir2.text = "...";
	tbOpenDir2.clicked.connect(leDirectory2, onOpenDir); 
	gbOpt.add(leDirectory2, tbOpenDir2);

	var leFileMask2 = new LineEdit;
	leFileMask2.label = "Path Mask:";
	leFileMask2.text = settings.value(fileMaskKey + "2", "");
	leFileMask2.toolTip = "Examples:\nPRJ ??????/\*.mol/fid\n\SMPL-*/10/pdata/\*.mol";
	gbOpt.add(leFileMask2);
	
	if(!dialog.exec()) 
		return;

	print("Directory: " + dirName);
	print("File Mask: " + leFileMask.text);
		
	dirName = leDirectory.text;
	var dir = new Dir(dirName);
	if (!dirName || !dir.exists)
	{
		MessageBox.critical("Directory " + dirName + " does not exist.", MessageBox.Ok);
		return;
	}

	dirName2 = leDirectory2.text;
	var dir2 = new Dir(dirName2);
	if (gbOpt.checked && (!dirName2 || !dir2.exists))
	{
		MessageBox.critical("Directory " + dirName2 + " does not exist.", MessageBox.Ok);
		return;
	}

	settings.setValue(dirSettingsKey, dirName);
	settings.setValue(dirSettingsKey+"2", dirName2);
	settings.setValue(fileMaskKey, leFileMask.text);
	settings.setValue(fileMaskKey+"2", leFileMask2.text);
	settings.setValue(optFilesKey, gbOpt.checked);

	var mainFiles = getMaskFilesAdv(dir, leFileMask.text, Dir.Name, Dir.Name);
	print("Main Files Found: " + mainFiles);
	
	if (!mainFiles.length)
	{
		MessageBox.warning("No files matching to the specified main path mask found.", MessageBox.Ok);
		return;
	}
	
	var optFiles = new Array();
	if (gbOpt.checked)
	{
		optFiles = getMaskFilesAdv(dir2, leFileMask2.text, Dir.Name, Dir.Name);
		print("Optional Files Found: " + optFiles);
	}

	var allFiles = new Array();
	for(var i = 0; i < Math.max(mainFiles.length, optFiles.length); i++)
	{
		if (i < mainFiles.length)
			allFiles.push(mainFiles[i]);
		if (i < optFiles.length)
			allFiles.push(optFiles[i]);
	}
	
	oldFileHistory = serialization.fileHistory;
	serialization.fileHistory = false;
	serialization.open(allFiles);
	serialization.fileHistory = oldFileHistory;
}
